@@ -3,9 +3,9 @@ ConvertType - Convert between various types of files.
 ConvertType is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -29,16 +29,18 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <string.h>
 #include <stdlib.h>
 
+#include <gnuastro/eps.h>
+#include <gnuastro/pdf.h>
 #include <gnuastro/txt.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/jpeg.h>
 #include <gnuastro/arithmetic.h>
 
 #include <gnuastro-internal/timing.h>
+#include <gnuastro-internal/checkset.h>
 
 #include "main.h"
-
-#include "eps.h"
-#include "jpeg.h"
+#include "color.h"
 
 
 
@@ -70,16 +72,15 @@ convertt_change(struct converttparams *p)
       {
         /* Make a condition array: all pixels with a value equal to
            `change->from' will be set as 1 in this array. */
-        cond=gal_arithmetic(GAL_ARITHMETIC_OP_EQ, GAL_ARITHMETIC_NUMOK,
+        cond=gal_arithmetic(GAL_ARITHMETIC_OP_EQ, 1, GAL_ARITHMETIC_NUMOK,
                             channel, change->from);
 
         /* Now, use the condition array to set the proper values. */
-        channel=gal_arithmetic(GAL_ARITHMETIC_OP_WHERE, flags, channel,
+        channel=gal_arithmetic(GAL_ARITHMETIC_OP_WHERE, 1, flags, channel,
                                cond, change->to);
 
         /* Clean up, since we set the free flag, all extra arrays have been
            freed.*/
-        gal_data_free(cond);
         gal_data_free(change->from);
       }
 
@@ -103,11 +104,11 @@ convertt_trunc_function(int operator, gal_data_t *data, gal_data_t *value)
 
   /* Make a condition array: all pixels with a value equal to
      `change->from' will be set as 1 in this array. */
-  cond=gal_arithmetic(operator, GAL_ARITHMETIC_NUMOK, data, value);
+  cond=gal_arithmetic(operator, 1, GAL_ARITHMETIC_NUMOK, data, value);
 
 
   /* Now, use the condition array to set the proper values. */
-  out=gal_arithmetic(GAL_ARITHMETIC_OP_WHERE, flags, data, cond, value);
+  out=gal_arithmetic(GAL_ARITHMETIC_OP_WHERE, 1, flags, data, cond, value);
 
 
   /* A small sanity check. The process must be in-place so the original
@@ -164,56 +165,6 @@ convertt_truncate(struct converttparams *p)
 
 
 
-/**************************************************************/
-/**************       Save text and FITS        ***************/
-/**************************************************************/
-static void
-save_with_gnuastro_lib(struct converttparams *p)
-{
-  gal_data_t *channel;
-
-  /* Determine the type. */
-  switch(p->outformat)
-    {
-
-    /* FITS: a FITS file can have many extensions (channels). */
-    case OUT_FORMAT_FITS:
-      for(channel=p->chll; channel!=NULL; channel=channel->next)
-        gal_fits_img_write(channel, p->cp.output, NULL, PROGRAM_STRING);
-      break;
-
-    /* Plain text: only one channel is acceptable. */
-    case OUT_FORMAT_TXT:
-      gal_txt_write(p->chll, NULL, p->cp.output, p->cp.dontdelete);
-      break;
-
-
-    /* Not recognized. */
-    default:
-      error(EXIT_FAILURE, 0, "%s: a bug! output format code `%d' not "
-            "recognized", __func__, p->outformat);
-    }
-}
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
-
 /**************************************************************/
 /**************       convert to 8 bit          ***************/
 /**************************************************************/
@@ -225,7 +176,6 @@ convertt_scale_to_uchar(struct converttparams *p)
   gal_data_t *channel, *prev, *copied, *mind, *maxd;
   float *f, *ff, m, tmin, tmax, min=FLT_MAX, max=-FLT_MAX;
 
-
   /* Convert everything to single precision floating point type and find
      the minimum and maximum values of all the channels in the process. */
   prev=NULL;
@@ -250,8 +200,8 @@ convertt_scale_to_uchar(struct converttparams *p)
             }
 
           /* Calculate the minimum and maximum. */
-          mind = gal_arithmetic(GAL_ARITHMETIC_OP_MINVAL, 0, channel);
-          maxd = gal_arithmetic(GAL_ARITHMETIC_OP_MAXVAL, 0, channel);
+          mind = gal_arithmetic(GAL_ARITHMETIC_OP_MINVAL, 1, 0, channel);
+          maxd = gal_arithmetic(GAL_ARITHMETIC_OP_MAXVAL, 1, 0, channel);
           tmin = *((float *)(mind->array));
           tmax = *((float *)(maxd->array));
           gal_data_free(mind);
@@ -272,14 +222,14 @@ convertt_scale_to_uchar(struct converttparams *p)
      maximum and minimum values in the image.*/
   if(p->fluxlow || p->fluxhigh)
     {
-      if(p->flminbyte)
+      if(p->forcemin)
         {
           /* Convert the fluxlow value to float and put it in min. */
           copied=gal_data_copy_to_new_type(p->fluxlow, GAL_TYPE_FLOAT32);
           min = *((float *)(copied->array));
           gal_data_free(copied);
         }
-      if(p->fhmaxbyte)
+      if(p->forcemax)
         {
           /* Convert the fluxhigh value to float and put it in min. */
           copied=gal_data_copy_to_new_type(p->fluxhigh, GAL_TYPE_FLOAT32);
@@ -290,7 +240,6 @@ convertt_scale_to_uchar(struct converttparams *p)
   m=(float)maxbyte/(max-min);
 
 
-
   /* Convert all the non-blank channels to unsigned char. */
   prev=NULL;
   for(channel=p->chll; channel!=NULL; channel=channel->next)
@@ -332,7 +281,6 @@ convertt_scale_to_uchar(struct converttparams *p)
 
       /* Set the prev pointer. */
       prev=channel;
-
     }
 }
 
@@ -354,12 +302,14 @@ convertt_scale_to_uchar(struct converttparams *p)
 
 
 
+
 /**************************************************************/
 /**************           Main function         ***************/
 /**************************************************************/
 void
 convertt(struct converttparams *p)
 {
+  gal_data_t *channel;
 
   /* Make any of the desired changes to the data. */
   if(p->changeaftertrunc)
@@ -373,35 +323,57 @@ convertt(struct converttparams *p)
       convertt_truncate(p);
     }
 
-
+  /* Convert a mono/single color channel to a color format. */
+  if(p->colormap)
+    switch(p->colormap->status)
+      {
+      case COLOR_HSV:  color_from_mono_hsv(p);     break;
+      case COLOR_SLS:  color_from_mono_sls(p);     break;
+      case COLOR_GRAY: convertt_scale_to_uchar(p); break;
+      default:
+        error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+              "the problem. The value %d is not a recognized color-space "
+              "code", __func__, PACKAGE_BUGREPORT, p->colormap->status);
+      }
 
   /* Save the outputs: */
   switch(p->outformat)
     {
-    case OUT_FORMAT_TXT:
+    /* FITS: a FITS file can have many extensions (channels). */
     case OUT_FORMAT_FITS:
-      save_with_gnuastro_lib(p);
+      if(p->numch==3 && p->rgbtohsv)
+        color_rgb_to_hsv(p);
+      for(channel=p->chll; channel!=NULL; channel=channel->next)
+        gal_fits_img_write(channel, p->cp.output, NULL, PROGRAM_NAME);
       break;
 
+    /* Plain text: only one channel is acceptable. */
+    case OUT_FORMAT_TXT:
+      gal_checkset_writable_remove(p->cp.output, 0, p->cp.dontdelete);
+      gal_txt_write(p->chll, NULL, p->cp.output, 0);
+      break;
+
+    /* JPEG: */
     case OUT_FORMAT_JPEG:
-#ifdef HAVE_LIBJPEG
-      convertt_scale_to_uchar(p);
-      jpeg_write(p);
-#else
-      error(EXIT_FAILURE, 0, "you have asked for a JPEG output, however, "
-            "when %s was configured libjpeg was not available. To write "
-            "to JPEG files, libjpeg is required. Please install it and "
-            "configure, make and install %s again", PACKAGE_STRING,
-            PACKAGE_STRING);
-#endif
+      if(!p->colormap) convertt_scale_to_uchar(p);
+      gal_jpeg_write(p->chll, p->cp.output, p->quality, p->widthincm);
       break;
 
+    /* EPS. */
     case OUT_FORMAT_EPS:
-    case OUT_FORMAT_PDF:
-      convertt_scale_to_uchar(p);
-      eps_write_eps_or_pdf(p);
+      if(!p->colormap) convertt_scale_to_uchar(p);
+      gal_eps_write(p->chll, p->cp.output, p->widthincm, p->borderwidth,
+                    p->hex, p->forcemin || p->forcemax, 0);
       break;
 
+    /* PDF */
+    case OUT_FORMAT_PDF:
+      if(!p->colormap) convertt_scale_to_uchar(p);
+      gal_pdf_write(p->chll, p->cp.output, p->widthincm, p->borderwidth,
+                    p->forcemin || p->forcemax);
+      break;
+
+    /* Not recognized. */
     default:
       error(EXIT_FAILURE, 0, "%s: a bug! Please contact us so we can find "
             "the problem and fix it The internal type of the output is "
