@@ -1,11 +1,11 @@
 /*********************************************************************
-NoiseChisel - Detect and segment signal in a noisy dataset.
+NoiseChisel - Detect signal in a noisy dataset.
 NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -38,7 +38,7 @@ struct argp_option program_options[] =
       UI_KEY_KERNEL,
       "STR",
       0,
-      "Filename of Kernel to convolve with input",
+      "Filename of kernel to convolve with input",
       GAL_OPTIONS_GROUP_INPUT,
       &p->kernelname,
       GAL_TYPE_STRING,
@@ -51,7 +51,7 @@ struct argp_option program_options[] =
       UI_KEY_KHDU,
       "STR",
       0,
-      "HDU containing Kernel image.",
+      "HDU containing kernel image.",
       GAL_OPTIONS_GROUP_INPUT,
       &p->khdu,
       GAL_TYPE_STRING,
@@ -60,44 +60,59 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "skysubtracted",
-      UI_KEY_SKYSUBTRACTED,
+      "convolved",
+      UI_KEY_CONVOLVED,
+      "STR",
       0,
-      0,
-      "Input is Sky subtracted (for error estimation).",
+      "Convolved image file to avoid convolution.",
       GAL_OPTIONS_GROUP_INPUT,
-      &p->skysubtracted,
-      GAL_OPTIONS_NO_ARG_TYPE,
-      GAL_OPTIONS_RANGE_0_OR_1,
+      &p->convolvedname,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
-      "minskyfrac",
-      UI_KEY_MINSKYFRAC,
-      "FLT",
+      "chdu",
+      UI_KEY_CHDU,
+      "STR",
       0,
-      "Min. fraction of undetected area in tile.",
+      "HDU/extension of convolved image in file.",
       GAL_OPTIONS_GROUP_INPUT,
-      &p->minskyfrac,
-      GAL_TYPE_FLOAT32,
-      GAL_OPTIONS_RANGE_GE_0_LE_1,
-      GAL_OPTIONS_MANDATORY,
+      &p->chdu,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
-      "minnumfalse",
-      UI_KEY_MINNUMFALSE,
-      "INT",
+      "widekernel",
+      UI_KEY_WIDEKERNEL,
+      "STR",
       0,
-      "Minimum number for S/N estimation.",
+      "Filename of wider kernel for better qthresh",
       GAL_OPTIONS_GROUP_INPUT,
-      &p->minnumfalse,
-      GAL_TYPE_SIZE_T,
-      GAL_OPTIONS_RANGE_GT_0,
-      GAL_OPTIONS_MANDATORY,
+      &p->widekernelname,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "whdu",
+      UI_KEY_WHDU,
+      "STR",
+      0,
+      "HDU containing wide kernel image.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &p->whdu,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+
+
 
 
     /* Tessellation. */
@@ -118,28 +133,43 @@ struct argp_option program_options[] =
 
 
 
+
+
     /* Output options. */
     {
-      "onlydetection",
-      UI_KEY_ONLYDETECTION,
+      "ignoreblankintiles",
+      UI_KEY_IGNOREBLANKINTILES,
       0,
       0,
-      "Stop at the end of detection.",
+      "Don't write input's blanks in tiled output.",
       GAL_OPTIONS_GROUP_OUTPUT,
-      &p->onlydetection,
+      &p->ignoreblankintiles,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
-      "grownclumps",
-      UI_KEY_GROWNCLUMPS,
+      "rawoutput",
+      UI_KEY_RAWOUTPUT,
       0,
       0,
-      "Save grown clumps instead of original.",
+      "Output only detection labels & 1-elem/tile grid.",
       GAL_OPTIONS_GROUP_OUTPUT,
-      &p->grownclumps,
+      &p->rawoutput,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "label",
+      UI_KEY_LABEL,
+      0,
+      0,
+      "Label/count detected pixels that are connected.",
+      GAL_OPTIONS_GROUP_OUTPUT,
+      &p->label,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
@@ -153,29 +183,16 @@ struct argp_option program_options[] =
     {
       0, 0, 0, 0,
       "Detection:",
-      ARGS_GROUP_DETECTION
+      UI_GROUP_DETECTION
     },
     {
-      "mirrordist",
-      UI_KEY_MIRRORDIST,
+      "meanmedqdiff",
+      UI_KEY_MEANMEDQDIFF,
       "FLT",
       0,
-      "Max. dist. (error multip.) to find mode.",
-      ARGS_GROUP_DETECTION,
-      &p->mirrordist,
-      GAL_TYPE_FLOAT32,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "modmedqdiff",
-      UI_KEY_MODMEDQDIFF,
-      "FLT",
-      0,
-      "Max. mode and median quant diff. per tile.",
-      ARGS_GROUP_DETECTION,
-      &p->modmedqdiff,
+      "Max. mean and median quant diff. per tile.",
+      UI_GROUP_DETECTION,
+      &p->meanmedqdiff,
       GAL_TYPE_FLOAT32,
       GAL_OPTIONS_RANGE_GE_0,
       GAL_OPTIONS_MANDATORY,
@@ -187,20 +204,47 @@ struct argp_option program_options[] =
       "FLT",
       0,
       "Quantile threshold on convolved image.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->qthresh,
       GAL_TYPE_FLOAT32,
       GAL_OPTIONS_RANGE_GE_0_LT_1,
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "outliersclip",
+      UI_KEY_OUTLIERSCLIP,
+      "FLT,FLT",
+      0,
+      "Sigma-clip params for qthresh outliers.",
+      UI_GROUP_DETECTION,
+      &p->outliersclip,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_read_sigma_clip
+    },
+    {
+      "outliersigma",
+      UI_KEY_OUTLIERSIGMA,
+      "FLT",
+      0,
+      "Multiple of sigma to define outliers.",
+      UI_GROUP_DETECTION,
+      &p->outliersigma,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "smoothwidth",
       UI_KEY_SMOOTHWIDTH,
       "INT",
       0,
       "Flat kernel width to smooth interpolated.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->smoothwidth,
       GAL_TYPE_SIZE_T,
       GAL_OPTIONS_RANGE_0_OR_ODD,
@@ -213,20 +257,33 @@ struct argp_option program_options[] =
       0,
       0,
       "Save quantile threshold estimation in file.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->checkqthresh,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "blankasforeground",
+      UI_KEY_BLANKASFOREGROUND,
+      0,
+      0,
+      "Blanks are foreground in erosion and opening.",
+      UI_GROUP_DETECTION,
+      &p->blankasforeground,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "erode",
       UI_KEY_ERODE,
       "INT",
       0,
       "Number of erosions after thresholding.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->erode,
       GAL_TYPE_SIZE_T,
       GAL_OPTIONS_RANGE_GE_0,
@@ -239,7 +296,7 @@ struct argp_option program_options[] =
       "INT",
       0,
       "4 or 8 connectivity in erosion.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->erodengb,
       GAL_TYPE_SIZE_T,
       GAL_OPTIONS_RANGE_GT_0,
@@ -252,7 +309,7 @@ struct argp_option program_options[] =
       "FLT",
       0,
       "Quantile for no erosion.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->noerodequant,
       GAL_TYPE_FLOAT32,
       GAL_OPTIONS_RANGE_GE_0_LE_1,
@@ -265,10 +322,10 @@ struct argp_option program_options[] =
       "INT",
       0,
       "Depth of opening after erosion.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->opening,
       GAL_TYPE_SIZE_T,
-      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_RANGE_GE_0,
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
@@ -278,20 +335,59 @@ struct argp_option program_options[] =
       "INT",
       0,
       "4 or 8 connectivity in opening.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->openingngb,
       GAL_TYPE_SIZE_T,
       GAL_OPTIONS_RANGE_GT_0,
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "skyfracnoblank",
+      UI_KEY_SKYFRACNOBLANK,
+      0,
+      0,
+      "No blanks in tile undetected frac. (minskyfrac).",
+      UI_GROUP_DETECTION,
+      &p->skyfracnoblank,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "minskyfrac",
+      UI_KEY_MINSKYFRAC,
+      "FLT",
+      0,
+      "Min. fraction of undetected area in tile.",
+      UI_GROUP_DETECTION,
+      &p->minskyfrac,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GE_0_LE_1,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "checkdetsky",
+      UI_KEY_CHECKDETSKY,
+      0,
+      0,
+      "Save Sky value estimation for pseudo-dets.",
+      UI_GROUP_DETECTION,
+      &p->checkdetsky,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
     {
       "sigmaclip",
       UI_KEY_SIGMACLIP,
       "FLT,FLT",
       0,
       "Sigma multiple and, tolerance or number.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->sigmaclip,
       GAL_TYPE_STRING,
       GAL_OPTIONS_RANGE_ANY,
@@ -299,26 +395,13 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET,
       gal_options_read_sigma_clip
     },
-    {
-      "checkdetsky",
-      UI_KEY_CHECKDETSKY,
-      0,
-      0,
-      "Save Sky value estimation for pseudo-dets.",
-      ARGS_GROUP_DETECTION,
-      &p->checkdetsky,
-      GAL_OPTIONS_NO_ARG_TYPE,
-      GAL_OPTIONS_RANGE_0_OR_1,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
     {
       "dthresh",
       UI_KEY_DTHRESH,
       "FLT",
       0,
       "Sigma threshold for Pseudo-detections.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->dthresh,
       GAL_TYPE_FLOAT32,
       GAL_OPTIONS_RANGE_ANY,
@@ -326,78 +409,156 @@ struct argp_option program_options[] =
       GAL_OPTIONS_NOT_SET
     },
     {
-      "detsnminarea",
-      UI_KEY_DETSNMINAREA,
+      "dopening",
+      UI_KEY_DOPENING,
+      "INT",
+      0,
+      "Depth of opening after dthresh.",
+      UI_GROUP_DETECTION,
+      &p->dopening,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "dopeningngb",
+      UI_KEY_DOPENINGNGB,
+      "INT",
+      0,
+      "4 or 8 connectivity for dthresh opening.",
+      UI_GROUP_DETECTION,
+      &p->dopeningngb,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "holengb",
+      UI_KEY_HOLENGB,
+      "INT",
+      0,
+      "4 or 8 connectivity for filling holes.",
+      UI_GROUP_DETECTION,
+      &p->holengb,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "pseudoconcomp",
+      UI_KEY_PSEUDOCONCOMP,
+      "INT",
+      0,
+      "4 or 8 neighbors for labeling pseudo-dets.",
+      UI_GROUP_DETECTION,
+      &p->pseudoconcomp,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "snminarea",
+      UI_KEY_SNMINAREA,
       "INT",
       0,
       "Min. pseudo-detection area for S/N dist.",
-      ARGS_GROUP_DETECTION,
-      &p->detsnminarea,
+      UI_GROUP_DETECTION,
+      &p->snminarea,
       GAL_TYPE_SIZE_T,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
-      "checkdetsn",
-      UI_KEY_CHECKDETSN,
+      "checksn",
+      UI_KEY_CHECKSN,
       0,
       0,
       "Save pseudo-detection S/N values to a file.",
-      ARGS_GROUP_DETECTION,
-      &p->checkdetsn,
+      UI_GROUP_DETECTION,
+      &p->checksn,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
-      "detquant",
-      UI_KEY_DETQUANT,
+      "minnumfalse",
+      UI_KEY_MINNUMFALSE,
+      "INT",
+      0,
+      "Minimum number for S/N estimation.",
+      UI_GROUP_DETECTION,
+      &p->minnumfalse,
+      GAL_TYPE_SIZE_T,
+      GAL_OPTIONS_RANGE_GT_0,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "snquant",
+      UI_KEY_SNQUANT,
       "FLT",
       0,
       "Quantile in pseudo-det. to define true.",
-      ARGS_GROUP_DETECTION,
-      &p->detquant,
+      UI_GROUP_DETECTION,
+      &p->snquant,
       GAL_TYPE_FLOAT32,
-      GAL_OPTIONS_RANGE_GT_0_LT_1,
+      GAL_OPTIONS_RANGE_GE_0_LE_1,
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
-      "dilate",
-      UI_KEY_DILATE,
+      "snthresh",
+      UI_KEY_SNTHRESH,
+      "FLT",
+      0,
+      "Manually input pseudo-det S/N threshold.",
+      UI_GROUP_DETECTION,
+      &p->snthresh,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "detgrowquant",
+      UI_KEY_DETGROWQUANT,
+      "FLT",
+      0,
+      "Minimum quant. to expand true detections.",
+      UI_GROUP_DETECTION,
+      &p->detgrowquant,
+      GAL_TYPE_FLOAT32,
+      GAL_OPTIONS_RANGE_GE_0_LE_1,
+      GAL_OPTIONS_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
+    {
+      "detgrowmaxholesize",
+      UI_KEY_DETGROWMAXHOLESIZE,
       "INT",
       0,
-      "Number of times to dilate true detections.",
-      ARGS_GROUP_DETECTION,
-      &p->dilate,
+      "Max. area of holes after growth to fill.",
+      UI_GROUP_DETECTION,
+      &p->detgrowmaxholesize,
       GAL_TYPE_SIZE_T,
-      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_RANGE_GE_0,
       GAL_OPTIONS_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
     {
-      "dilatengb",
-      UI_KEY_DILATENGB,
-      "INT",
-      0,
-      "4 or 8 connectivity in final dilation.",
-      ARGS_GROUP_DETECTION,
-      &p->dilatengb,
-      GAL_TYPE_SIZE_T,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "cleandilated",
-      UI_KEY_CLEANDILATED,
+      "cleangrowndet",
+      UI_KEY_CLEANGROWNDET,
       0,
       0,
-      "Remove small S/N dilated objects.",
-      ARGS_GROUP_DETECTION,
-      &p->cleandilated,
+      "Remove small S/N grown detections.",
+      UI_GROUP_DETECTION,
+      &p->cleangrowndet,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_ANY,
       GAL_OPTIONS_NOT_MANDATORY,
@@ -409,7 +570,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Save all the detection steps to a file.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->checkdetection,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -422,7 +583,7 @@ struct argp_option program_options[] =
       0,
       0,
       "Final sky and its STD steps in a file.",
-      ARGS_GROUP_DETECTION,
+      UI_GROUP_DETECTION,
       &p->checksky,
       GAL_OPTIONS_NO_ARG_TYPE,
       GAL_OPTIONS_RANGE_0_OR_1,
@@ -432,118 +593,9 @@ struct argp_option program_options[] =
 
 
 
-    /* Segmentation */
-    {
-      0, 0, 0, 0,
-      "Segmentation:",
-      ARGS_GROUP_SEGMENTATION
-    },
-    {
-      "segsnminarea",
-      UI_KEY_SEGSNMINAREA,
-      "INT",
-      0,
-      "Minimum area of clumps for S/N estimation.",
-      ARGS_GROUP_SEGMENTATION,
-      &p->segsnminarea,
-      GAL_TYPE_SIZE_T,
-      GAL_OPTIONS_RANGE_GT_0,
-      GAL_OPTIONS_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "checkclumpsn",
-      UI_KEY_CHECKCLUMPSN,
-      0,
-      0,
-      "Save Sky clump S/N values into a file.",
-      ARGS_GROUP_SEGMENTATION,
-      &p->checkclumpsn,
-      GAL_OPTIONS_NO_ARG_TYPE,
-      GAL_OPTIONS_RANGE_0_OR_1,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "segquant",
-      UI_KEY_SEGQUANT,
-      "FLT",
-      0,
-      "S/N Quantile of true sky clumps.",
-      ARGS_GROUP_SEGMENTATION,
-      &p->segquant,
-      GAL_TYPE_FLOAT32,
-      GAL_OPTIONS_RANGE_GT_0,
-      GAL_OPTIONS_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "keepmaxnearriver",
-      UI_KEY_KEEPMAXNEARRIVER,
-      0,
-      0,
-      "Keep clumps with peak touching a river.",
-      ARGS_GROUP_SEGMENTATION,
-      &p->keepmaxnearriver,
-      GAL_OPTIONS_NO_ARG_TYPE,
-      GAL_OPTIONS_RANGE_0_OR_1,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "gthresh",
-      UI_KEY_GTHRESH,
-      "FLT",
-      0,
-      "Multiple of STD to stop growing clumps.",
-      ARGS_GROUP_SEGMENTATION,
-      &p->gthresh,
-      GAL_TYPE_FLOAT32,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "minriverlength",
-      UI_KEY_MINRIVERLENGTH,
-      "INT",
-      0,
-      "Minimum len of useful grown clump rivers.",
-      ARGS_GROUP_SEGMENTATION,
-      &p->minriverlength,
-      GAL_TYPE_SIZE_T,
-      GAL_OPTIONS_RANGE_ANY,
-      GAL_OPTIONS_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "objbordersn",
-      UI_KEY_OBJBORDERSN,
-      "FLT",
-      0,
-      "Min. S/N for grown clumps as one object.",
-      ARGS_GROUP_SEGMENTATION,
-      &p->objbordersn,
-      GAL_TYPE_FLOAT32,
-      GAL_OPTIONS_RANGE_GE_0,
-      GAL_OPTIONS_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
-    {
-      "checksegmentation",
-      UI_KEY_CHECKSEGMENTATION,
-      0,
-      0,
-      "Store segmentation steps in a file.",
-      ARGS_GROUP_SEGMENTATION,
-      &p->checksegmentation,
-      GAL_OPTIONS_NO_ARG_TYPE,
-      GAL_OPTIONS_RANGE_0_OR_1,
-      GAL_OPTIONS_NOT_MANDATORY,
-      GAL_OPTIONS_NOT_SET
-    },
 
 
+    /* Operating mode options. */
     {
       "continueaftercheck",
       UI_KEY_CONTINUEAFTERCHECK,
@@ -560,6 +612,9 @@ struct argp_option program_options[] =
 
 
 
+
+
+    /* End of options. */
     {0}
   };
 
