@@ -5,7 +5,7 @@ NoiseChisel is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -30,6 +30,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro/fits.h>
 #include <gnuastro/label.h>
+#include <gnuastro/blank.h>
 #include <gnuastro/binary.h>
 #include <gnuastro/threads.h>
 #include <gnuastro/pointer.h>
@@ -55,8 +56,10 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 void
 detection_initial(struct noisechiselparams *p)
 {
+  float *f;
   char *msg;
   uint8_t *b, *bf;
+  int resetblank=0;
   struct timeval t0, t1;
 
 
@@ -78,13 +81,22 @@ detection_initial(struct noisechiselparams *p)
     }
 
 
+  /* Remove any blank elements from the binary image if requested. */
+  if(p->blankasforeground==0 && gal_blank_present(p->binary,0))
+    {
+      resetblank=1;
+      bf=(b=p->binary->array)+p->binary->size;
+      do *b = *b==GAL_BLANK_UINT8 ? 0 : *b; while(++b<bf);
+    }
+
+
   /* Erode the image. */
   if(!p->cp.quiet) gettimeofday(&t1, NULL);
   gal_binary_erode(p->binary, p->erode, p->erodengb==4 ? 1 : 2, 1);
   if(!p->cp.quiet)
     {
-      if( asprintf(&msg, "Eroded %zu time%s (%zu-connectivity).", p->erode,
-                   p->erode>1?"s":"", p->erodengb)<0 )
+      if( asprintf(&msg, "Eroded %zu time%s (%zu-connected).", p->erode,
+                   p->erode!=1?"s":"", p->erodengb)<0 )
         error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t1, msg, 2);
       free(msg);
@@ -107,14 +119,23 @@ detection_initial(struct noisechiselparams *p)
   gal_binary_open(p->binary, p->opening, p->openingngb==4 ? 1 : 2, 1);
   if(!p->cp.quiet)
     {
-      if( asprintf(&msg, "Opened (depth: %zu, %s connectivity).",
-                   p->opening, p->openingngb==4 ? "4" : "8")<0 )
+      if( asprintf(&msg, "Opened (depth: %zu, %zu-connected).",
+                   p->opening, p->openingngb)<0 )
         error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       gal_timing_report(&t1, msg, 2);
       free(msg);
     }
 
 
+  /* Reset the blank values (if requested). */
+  if(resetblank)
+    {
+      f=p->input->array;
+      bf=(b=p->binary->array)+p->binary->size;
+      do *b = isnan(*f++) ? GAL_BLANK_UINT8 : *b; while(++b<bf);
+    }
+
+
   /* Label the connected components. */
   p->numinitialdets=gal_binary_connected_components(p->binary, &p->olabel,
                                                     p->binary->ndim);
@@ -218,6 +239,7 @@ detection_fill_holes_open(void *in_prm)
   struct noisechiselparams *p=fho_prm->p;
 
   void *tarray;
+  uint8_t *b, *bf;
   gal_data_t *tile, *copy, *tblock;
   size_t i, dsize[]={1,1,1,1,1,1,1,1,1,1}; /* For upto 10-Dimensions! */
 
@@ -245,12 +267,23 @@ detection_fill_holes_open(void *in_prm)
       /* Copy the tile into the contiguous patch of memory to work on, but
          first reset the size element so `gal_data_copy_to_allocated' knows
          there is enough space. */
+      copy->flag=0;
       copy->size=p->maxltcontig;
       gal_data_copy_to_allocated(tile, copy);
 
+      /* Take blank values to the background (set them to zero) if
+         necsesary. */
+      if( p->blankasforeground==0
+          && gal_blank_present(p->input,0)
+          && gal_blank_present(copy, 1) )
+        {
+          bf=(b=copy->array)+copy->size;
+          do *b = *b==GAL_BLANK_UINT8 ? 0 : *b; while(++b<bf);
+        }
+
       /* Fill the holes in this tile: holes with maximal connectivity means
          that they are most strongly bounded. */
-      gal_binary_holes_fill(copy, copy->ndim, -1);
+      gal_binary_holes_fill(copy, p->holengb==4 ? 1 : 2, -1);
       if(fho_prm->step==1)
         {
           detection_write_in_large(tile, copy);
@@ -260,7 +293,7 @@ detection_fill_holes_open(void *in_prm)
         }
 
       /* Open all the regions. */
-      gal_binary_open(copy, 1, 1, 1);
+      gal_binary_open(copy, p->dopening, p->dopeningngb==4 ? 1 : 2, 1);
 
       /* Write the copied region back into the large input and AFTERWARDS,
          correct the tile's pointers, the pointers must not be corrected
@@ -291,7 +324,10 @@ static size_t
 detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
                       gal_data_t *worklab, int s0d1)
 {
+  float *f;
+  uint8_t *b, *bf;
   gal_data_t *bin;
+  int con=p->pseudoconcomp==4 ? 1 : 2;
   struct fho_params fho_prm={0, NULL, workbin, worklab, p};
 
 
@@ -339,6 +375,14 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
           gal_threads_spin_off(detection_fill_holes_open, &fho_prm,
                                p->ltl.tottiles, p->cp.numthreads);
 
+          /* Reset the blank values (if they were changed). */
+          if( p->blankasforeground==0 && gal_blank_present(p->input,0) )
+            {
+              f=p->input->array;
+              bf=(b=workbin->array)+workbin->size;
+              do *b = isnan(*f++) ? GAL_BLANK_UINT8 : *b; while(++b<bf);
+            }
+
           /* Set the extension name based on the step. */
           switch(fho_prm.step)
             {
@@ -398,7 +442,7 @@ detection_pseudo_find(struct noisechiselparams *p, gal_data_t *workbin,
       do if(*b==GAL_BLANK_UINT8) *b = !s0d1; while(++b<bf);
     }
   */
-  return gal_binary_connected_components(workbin, &worklab, 1);
+  return gal_binary_connected_components(workbin, &worklab, con);
 }
 
 
@@ -410,7 +454,7 @@ static void
 detection_sn_write_to_file(struct noisechiselparams *p, gal_data_t *sn,
                            gal_data_t *snind, int s0d1D2)
 {
-  char *str;
+  char *str, *extname=NULL;
   gal_list_str_t *comments=NULL;
 
   /* Comment for extension on further explanation. */
@@ -430,11 +474,15 @@ detection_sn_write_to_file(struct noisechiselparams *p, gal_data_t *sn,
   gal_list_str_add(&comments, str, 1);
 
 
-  /* Set the file name. */
+  /* Set the file name and write the table. */
   str = ( s0d1D2
           ? ( s0d1D2==2 ? p->detsn_D_name : p->detsn_d_name )
           : p->detsn_s_name );
-  threshold_write_sn_table(p, sn, snind, str, comments);
+  if( p->cp.tableformat!=GAL_TABLE_FORMAT_TXT )
+    extname = ( s0d1D2
+                ? ( s0d1D2==2 ? "GROWN_DETECTION_SN" : "DET_PSEUDODET_SN" )
+                : "SKY_PSEUDODET_SN" );
+  threshold_write_sn_table(p, sn, snind, str, comments, extname);
   gal_list_str_free(comments, 1);
 
 
@@ -460,14 +508,13 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
   gal_data_t *sn, *snind;
   int32_t *plabend, *indarr=NULL;
   double ave, err, *xy, *brightness;
+  float s, ss, *f, *ff, *fs, *sky=p->sky->array;
   size_t ind, ndim=p->input->ndim, xyncols=1+ndim;
   size_t i, *area, counter=0, *dsize=p->input->dsize;
-  float *img=p->input->array, *f=p->input->array, *ff=f+p->input->size;
   int32_t *plab = worklab->array, *dlab = s0d1D2 ? NULL : p->olabel->array;
   size_t *coord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
                                       "coord");
 
-
   /* Sanity check. */
   if(p->input->type!=GAL_TYPE_FLOAT32)
     error(EXIT_FAILURE, 0, "%s: the input dataset must be float32 type, "
@@ -503,6 +550,8 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
                                   NULL) );
 
   /* Go over all the pixels and get the necessary information. */
+  fs = f = p->input->array;
+  ff = f + p->input->size;
   do
     {
       /* All this work is only necessary when we are actually on a
@@ -525,12 +574,15 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
 
           /* Save all the necessary values. */
           ++area[*plab];
-          brightness[*plab] += *f;
-          if( *f > 0.0f )  /* For calculatiing the approximate center, */
+          gal_dimension_index_to_coord(f-fs, ndim, dsize, coord);
+          s  = sky[ gal_tile_full_id_from_coord(&p->cp.tl, coord) ];
+          ss = *f-s;
+          brightness[*plab] += ss;
+          if( ss > 0.0f )  /* For calculatiing the approximate center, */
             {              /* necessary for calculating Sky and STD.   */
-              xy[*plab*xyncols  ] += *f;
-              xy[*plab*xyncols+1] += (double)((f-img)/dsize[1]) * *f;
-              xy[*plab*xyncols+2] += (double)((f-img)%dsize[1]) * *f;
+              xy[*plab*xyncols  ] += ss;
+              xy[*plab*xyncols+1] += (double)coord[0] * ss;
+              xy[*plab*xyncols+2] += (double)coord[1] * ss;
             }
         }
 
@@ -585,9 +637,7 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
           coord[0]=GAL_DIMENSION_FLT_TO_INT( xy[i*xyncols+1]/xy[i*xyncols] );
           coord[1]=GAL_DIMENSION_FLT_TO_INT( xy[i*xyncols+2]/xy[i*xyncols] );
 
-          /* Calculate the Sky and Sky standard deviation on this tile. */
-          ave -= ((float *)(p->sky->array))[
-                         gal_tile_full_id_from_coord(&p->cp.tl, coord) ];
+          /* Get the Sky standard deviation on this tile. */
           err  = ((float *)(p->std->array))[
                          gal_tile_full_id_from_coord(&p->cp.tl, coord) ];
 
@@ -612,6 +662,12 @@ detection_sn(struct noisechiselparams *p, gal_data_t *worklab, size_t num,
           }
     }
 
+
+  /* A small sanity check. */
+  if( s0d1D2==0 && counter==0 )
+    error(EXIT_FAILURE, 0, "no sky pseudo-detections.");
+
+
   /* If we are in Sky mode, the sizes have to be corrected */
   if(s0d1D2==0)
     {
@@ -706,36 +762,44 @@ detection_pseudo_real(struct noisechiselparams *p)
   workbin->flag=p->input->flag;
 
 
-  /* Over the Sky: find the pseudo-detections and make the S/N table. */
-  if(!p->cp.quiet) gettimeofday(&t1, NULL);
-  numpseudo=detection_pseudo_find(p, workbin, worklab, 0);
-  sn=detection_sn(p, worklab, numpseudo, 0, "PSEUDOS-FOR-SN");
-
-
-  /* A small sanity check */
-  if( sn->size < p->minnumfalse)
-    error(EXIT_FAILURE, 0, "only %zu pseudo-detections could be found over "
-          "the sky region to estimate an S/N. This is less than %zu (value "
-          "to `--minnumfalse' option). Please adjust parameters like "
-          "`--dthresh', `--snminarea', or make sure that there actually "
-          "is sufficient sky area after initial detection. You can use "
-          "`--checkdetection' to see every step until this point", sn->size,
-          p->minnumfalse);
-
-
-  /* Get the S/N quantile and report it if we are in non-quiet mode. */
-  quant=gal_statistics_quantile(sn, p->snquant, 1);
-  p->detsnthresh = *((float *)(quant->array));
-  if(!p->cp.quiet)
+  /* If a manual S/N is given, then don't bother with using
+     pseudo-detections. */
+  if( isnan(p->snthresh) )
     {
-      if( asprintf(&msg, "Pseudo-det S/N: %.3f (%g quant of %zu).",
-                   p->detsnthresh, p->snquant, sn->size)<0 )
-        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
-      gal_timing_report(&t1, msg, 2);
-      free(msg);
+      /* Over the Sky: find the pseudo-detections and make the S/N
+         table. */
+      if(!p->cp.quiet) gettimeofday(&t1, NULL);
+      numpseudo=detection_pseudo_find(p, workbin, worklab, 0);
+      sn=detection_sn(p, worklab, numpseudo, 0, "PSEUDOS-FOR-SN");
+
+
+      /* A small sanity check */
+      if( sn->size < p->minnumfalse)
+        error(EXIT_FAILURE, 0, "only %zu pseudo-detections could be found "
+              "over the sky region to estimate an S/N. This is less than "
+              "%zu (value to `--minnumfalse' option). Please adjust "
+              "parameters like `--dthresh', `--snminarea', or make sure "
+              "that there actually is sufficient sky area after initial "
+              "detection. You can use `--checkdetection' to see every step "
+              "until this point", sn->size, p->minnumfalse);
+
+
+      /* Get the S/N quantile and report it if we are in non-quiet mode. */
+      quant=gal_statistics_quantile(sn, p->snquant, 1);
+      p->detsnthresh = *((float *)(quant->array));
+      if(!p->cp.quiet)
+        {
+          if( asprintf(&msg, "Pseudo-det S/N: %.3f (%g quant of %zu).",
+                       p->detsnthresh, p->snquant, sn->size)<0 )
+            error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+          gal_timing_report(&t1, msg, 2);
+          free(msg);
+        }
+      gal_data_free(sn);
+      gal_data_free(quant);
     }
-  gal_data_free(sn);
-  gal_data_free(quant);
+  else
+    p->detsnthresh=p->snthresh;
 
 
   /* Over the detections: find pseudo-detections and make S/N table. */
@@ -769,6 +833,7 @@ detection_final_remove_small_sn(struct noisechiselparams *p,
   gal_data_t *sn, *snind;
   int32_t *l, *lf, curlab=1;
   gal_list_str_t *comments=NULL;
+  char *extname="GROWN_DETECTION_SN";
   int32_t *newlabs=gal_pointer_allocate(GAL_TYPE_INT32, num+1, 1, __func__,
                                         "newlabs");
 
@@ -816,9 +881,8 @@ detection_final_remove_small_sn(struct noisechiselparams *p,
       gal_list_str_add(&comments, "See also: `DILATED' "
                        "HDU of output with `--checkdetection'.", 1);
       gal_list_str_add(&comments, "S/N of finally grown detections.", 1);
-
-
-      threshold_write_sn_table(p, sn, snind, p->detsn_D_name, comments);
+      threshold_write_sn_table(p, sn, snind, p->detsn_D_name, comments,
+                               extname);
       gal_list_str_free(comments, 1);
 
     }
@@ -1098,6 +1162,13 @@ detection(struct noisechiselparams *p)
      pseudo-detections. */
   if(!p->cp.quiet) gettimeofday(&t1, NULL);
   num_true_initial=detection_remove_false_initial(p, workbin);
+  if(p->detectionname)
+    {
+      workbin->name="DETECTIONS-INIT-TRUE";
+      gal_fits_img_write(workbin, p->detectionname, NULL,
+                         PROGRAM_NAME);
+      workbin->name=NULL;
+    }
   if(!p->cp.quiet)
     {
       if( asprintf(&msg, "%zu false initial detections removed.",
