@@ -5,7 +5,7 @@ Segment is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -795,9 +795,8 @@ segment_save_sn_table(struct clumps_params *clprm)
   /* Set the column pointers and write them into a table.. */
   clumpinobj->next=sn;
   objind->next=clumpinobj;
-  gal_checkset_writable_remove(p->clumpsn_d_name, 0, 1);
   gal_table_write(objind, comments, p->cp.tableformat, p->clumpsn_d_name,
-                  "CLUMPS_SN", 0);
+                  "DET_CLUMP_SN", 0);
 
 
   /* Clean up. */
@@ -809,7 +808,9 @@ segment_save_sn_table(struct clumps_params *clprm)
 
   /* Abort NoiseChisel if necessary. */
   if(!p->continueaftercheck)
-    ui_abort_after_check(p, p->clumpsn_s_name, p->clumpsn_d_name,
+    ui_abort_after_check(p, p->clumpsn_s_name,
+                         ( p->cp.tableformat==GAL_TABLE_FORMAT_TXT
+                           ? p->clumpsn_d_name : NULL ),
                          "showing all clump S/N values");
 }
 
@@ -1121,6 +1122,11 @@ segment_output(struct segmentparams *p)
       p->std->name=NULL;
     }
 
+  /* Write the configuration keywords. */
+  gal_fits_key_write_filename("input", p->inputname, &p->cp.okeys, 1);
+  gal_fits_key_write_config(&p->cp.okeys, "Segment configuration",
+                            "SEGMENT-CONFIG", p->cp.output, "0");
+
   /* Let the user know that the output is written. */
   if(!p->cp.quiet)
     printf("  - Output written to `%s'.\n", p->cp.output);
@@ -1228,7 +1234,7 @@ segment(struct segmentparams *p)
 
 
   /* If the user wanted to check the segmentation and hasn't called
-     `continueaftercheck', then stop NoiseChisel. */
+     `continueaftercheck', then stop Segment. */
   if(p->segmentationname && !p->continueaftercheck)
     ui_abort_after_check(p, p->segmentationname, NULL,
                          "showing all segmentation steps");
