@@ -5,7 +5,7 @@ Statistics is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -92,10 +92,13 @@ enum option_keys_enum
   UI_KEY_MAXBINONE,
   UI_KEY_KHDU,
   UI_KEY_MIRRORDIST,
-  UI_KEY_MODMEDQDIFF,
+  UI_KEY_MEANMEDQDIFF,
+  UI_KEY_OUTLIERNUM,
+  UI_KEY_OUTLIERSIGMA,
+  UI_KEY_OUTLIERSCLIP,
   UI_KEY_SMOOTHWIDTH,
   UI_KEY_CHECKSKY,
-  UI_KEY_IGNOREBLANKINSKY,
+  UI_KEY_IGNOREBLANKINTILES,
   UI_KEY_SCLIPPARAMS,
 };
 
