@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2018, Free Software Foundation, Inc.
+Copyright (C) 2017-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -501,6 +501,17 @@ gal_convolve_spatial_general(gal_data_t *tiles, gal_data_t *kernel,
     error(EXIT_FAILURE, 0, "%s: only accepts `float32' type input and "
           "kernel currently", __func__);
 
+  /* It may happen that an input dataset is part of a linked list, but it
+     is not actually a tile structure (the user wants to convolve the whole
+     dataset without using tiles)! In that case, this function should break
+     beacuse a linked list is interpretted as a tile structure here.*/
+  if( tiles->block==NULL && tiles->next && tiles->next->block==NULL )
+    error(EXIT_FAILURE, 0, "%s: the input is a linked list but not a "
+          "tessellation (a list of tiles). This function is optimized to "
+          "work on a list of tiles. Please (temporarily) set the `next' "
+          "element of the input to `NULL' and call this function again",
+          __func__);
+
 
   /* Set the output datastructure.  */
   if(tocorrect) out=tocorrect;
@@ -559,6 +570,10 @@ gal_data_t *
 gal_convolve_spatial(gal_data_t *tiles, gal_data_t *kernel,
                      size_t numthreads, int edgecorrection, int convoverch)
 {
+  /* When there isn't any tile structure, `convoverch' must be set to
+     one. Recall that the input can be a single full dataset also. */
+  if(tiles->block==NULL) convoverch=1;
+
   /* Call the general function. */
   return gal_convolve_spatial_general(tiles, kernel, numthreads,
                                       edgecorrection, convoverch, NULL);
