@@ -6,7 +6,7 @@ IMPORTANT: This header must only be included the programs, not libraries.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017-2018, Free Software Foundation, Inc.
+Copyright (C) 2017-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -60,7 +60,7 @@ struct argp_option gal_commonopts_options[] =
       GAL_OPTIONS_KEY_SEARCHIN,
       "STR",
       0,
-      "Select column(s) in: `name', `unit', `comment'.",
+      "Select column(s): `name', `unit', `comment'.",
       GAL_OPTIONS_GROUP_INPUT,
       &cp->searchin,
       GAL_TYPE_STRING,
@@ -82,6 +82,19 @@ struct argp_option gal_commonopts_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "stdintimeout",
+      GAL_OPTIONS_KEY_STDINTIMEOUT,
+      "INT",
+      0,
+      "Micro-seconds to wait for standard input.",
+      GAL_OPTIONS_GROUP_INPUT,
+      &cp->stdintimeout,
+      GAL_TYPE_LONG,
+      GAL_OPTIONS_RANGE_GE_0,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET
+    },
 
 
 
@@ -185,6 +198,20 @@ struct argp_option gal_commonopts_options[] =
       GAL_OPTIONS_NOT_MANDATORY,
       GAL_OPTIONS_NOT_SET
     },
+    {
+      "interpmetric",
+      GAL_OPTIONS_KEY_INTERPMETRIC,
+      "INT",
+      0,
+      "Interpolation metric (radial, manhattan).",
+      GAL_OPTIONS_GROUP_TESSELLATION,
+      &cp->interpmetric,
+      GAL_TYPE_STRING,
+      GAL_OPTIONS_RANGE_ANY,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_read_interpmetric
+    },
     {
       "interpnumngb",
       GAL_OPTIONS_KEY_INTERPNUMNGB,
@@ -239,7 +266,7 @@ struct argp_option gal_commonopts_options[] =
       GAL_OPTIONS_KEY_TABLEFORMAT,
       "STR",
       0,
-      "Table format: `fits-ascii', `fits-binary'.",
+      "Table fmt: `fits-ascii', `fits-binary', `txt'.",
       GAL_OPTIONS_GROUP_OUTPUT,
       &cp->tableformat,
       GAL_TYPE_STRING,
@@ -381,6 +408,20 @@ struct argp_option gal_commonopts_options[] =
       GAL_OPTIONS_NOT_SET,
       gal_options_call_parse_config_file
     },
+    {
+      "checkconfig",
+      GAL_OPTIONS_KEY_CHECKCONFIG,
+      0,
+      0,
+      "List all config files and variables read.",
+      GAL_OPTIONS_GROUP_OPERATING_MODE,
+      &cp->checkconfig,
+      GAL_OPTIONS_NO_ARG_TYPE,
+      GAL_OPTIONS_RANGE_0_OR_1,
+      GAL_OPTIONS_NOT_MANDATORY,
+      GAL_OPTIONS_NOT_SET,
+      gal_options_check_config
+    },
     {
       "setdirconf",
       GAL_OPTIONS_KEY_SETDIRCONF,
