@@ -3,9 +3,9 @@ Functions to convert a FITS array to a C array and vice versa.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -37,6 +37,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 /* Include other headers if necessary here. Note that other header files
    must be included before the C++ preparations below */
 #include <math.h>
+#include <time.h>
 #include <float.h>
 
 #include <fitsio.h>
@@ -44,6 +45,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <wcslib/wcshdr.h>
 #include <wcslib/wcsfix.h>
 
+#include <gnuastro/list.h>
 #include <gnuastro/data.h>
 #include <gnuastro/table.h>
 
@@ -68,7 +70,7 @@ __BEGIN_C_DECLS  /* From C++ preparations */
 
 /* Macros. */
 #define GAL_FITS_MAX_NDIM 999
-
+#define GAL_FITS_KEY_TITLE_START "                      / "
 
 
 
@@ -159,9 +161,19 @@ gal_fits_hdu_open_format(char *filename, char *hdu, int img0_tab1);
 /**************************************************************/
 /**********            Header keywords             ************/
 /**************************************************************/
+void *
+gal_fits_key_img_blank(uint8_t type);
+
 void
 gal_fits_key_clean_str_value(char *string);
 
+char *
+gal_fits_key_date_to_struct_tm(char *fitsdate, struct tm *tp);
+
+size_t
+gal_fits_key_date_to_seconds(char *fitsdate, char **subsecstr,
+                             double *subsec);
+
 void
 gal_fits_key_read_from_ptr(fitsfile *fptr, gal_data_t *keysll,
                            int readcomment, int readunit);
@@ -180,19 +192,37 @@ gal_fits_key_list_add_end(gal_fits_list_key_t **list, uint8_t type,
                           char *keyname, int kfree, void *value, int vfree,
                           char *comment, int cfree, char *unit);
 
+void
+gal_fits_key_list_reverse(gal_fits_list_key_t **list);
+
+void
+gal_fits_key_write_title_in_ptr(char *title, fitsfile *fptr);
+
 void
 gal_fits_key_write_filename(char *keynamebase, char *filename,
-                            gal_fits_list_key_t **list);
+                            gal_fits_list_key_t **list, int top1end0);
 
 void
 gal_fits_key_write_wcsstr(fitsfile *fptr, char *wcsstr, int nkeyrec);
 
 void
-gal_fits_key_write(fitsfile *fptr, gal_fits_list_key_t **keylist);
+gal_fits_key_write(gal_fits_list_key_t **keylist, char *title,
+                   char *filename, char *hdu);
 
 void
-gal_fits_key_write_version(fitsfile *fptr, gal_fits_list_key_t *headers,
-                           char *program_string);
+gal_fits_key_write_in_ptr(gal_fits_list_key_t **keylist, fitsfile *fptr);
+
+void
+gal_fits_key_write_version(gal_fits_list_key_t **keylist, char *title,
+                           char *filename, char *hdu);
+
+void
+gal_fits_key_write_version_in_ptr(gal_fits_list_key_t **keylist, char *title,
+                                  fitsfile *fptr);
+
+void
+gal_fits_key_write_config(gal_fits_list_key_t **keylist, char *title,
+                          char *extname, char *filename, char *hdu);
 
 
 
@@ -202,13 +232,14 @@ gal_fits_key_write_version(fitsfile *fptr, gal_fits_list_key_t *headers,
  ******************     Array functions      *****************
  *************************************************************/
 void
-gal_fits_img_info(fitsfile *fptr, int *type, size_t *ndim, size_t **dsize);
+gal_fits_img_info(fitsfile *fptr, int *type, size_t *ndim, size_t **dsize,
+                  char **name, char **unit);
 
 gal_data_t *
 gal_fits_img_read(char *filename, char *hdu, size_t minmapsize);
 
 gal_data_t *
-gal_fits_img_read_to_type(char *inputname, char *inhdu, uint8_t type,
+gal_fits_img_read_to_type(char *inputname, char *hdu, uint8_t type,
                           size_t minmapsize);
 
 gal_data_t *
@@ -252,11 +283,11 @@ gal_fits_tab_info(char *filename, char *hdu, size_t *numcols,
 gal_data_t *
 gal_fits_tab_read(char *filename, char *hdu, size_t numrows,
                   gal_data_t *colinfo, gal_list_sizet_t *indexll,
-                  int minmapsize);
+                  size_t minmapsize);
 
 void
 gal_fits_tab_write(gal_data_t *cols, gal_list_str_t *comments,
-                   int tableformat, char *filename, int dontdelete);
+                   int tableformat, char *filename, char *extname);
 
 
 
