@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015-2018, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -167,22 +167,21 @@ gal_statistics_cfp(gal_data_t *data, gal_data_t *bins, int normalize);
 
 
 /****************************************************************
- *****************        Sigma clip         ********************
+ *****************         Outliers          ********************
  ****************************************************************/
 gal_data_t *
 gal_statistics_sigma_clip(gal_data_t *input, float multip, float param,
                           int inplace, int quiet);
 
+gal_data_t *
+gal_statistics_outlier_positive(gal_data_t *input, size_t window_size,
+                                float sigma, float sigclip_multip,
+                                float sigclip_param, int inplace, int quiet);
 
-
-
-
-/****************************************************************/
-/*************         Identify outliers         ****************/
-/****************************************************************/
-void
-gal_statistics_remove_outliers_flat_cdf(float *sorted, size_t *outsize);
-
+gal_data_t *
+gal_statistics_outlier_flat_cfp(gal_data_t *input, size_t dist, float thresh,
+                                float width, int inplace, int quiet,
+                                size_t *index);
 
 
 
