@@ -5,7 +5,7 @@ This is part of GNU Astronomy Utilities (Gnuastro) package.
 Original author:
      Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2018, Free Software Foundation, Inc.
+Copyright (C) 2018-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -246,7 +246,9 @@ gal_label_watershed(gal_data_t *values, gal_data_t *indexs,
     {
       gal_qsort_index_single=values->array;
       qsort(indexs->array, indexs->size, sizeof(size_t),
-            min0_max1 ? gal_qsort_index_single_d : gal_qsort_index_single_i);
+            ( min0_max1
+              ? gal_qsort_index_single_float32_d
+              : gal_qsort_index_single_float32_i) );
     }
 
 
