@@ -2,9 +2,9 @@
 Function to parse options and configuration file values.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2017, Free Software Foundation, Inc.
+Copyright (C) 2017-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -30,12 +30,15 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 
 #include <gnuastro/git.h>
 #include <gnuastro/txt.h>
+#include <gnuastro/fits.h>
 #include <gnuastro/list.h>
 #include <gnuastro/data.h>
 #include <gnuastro/table.h>
 #include <gnuastro/blank.h>
 #include <gnuastro/threads.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/arithmetic.h>
+#include <gnuastro/interpolate.h>
 
 #include <gnuastro-internal/timing.h>
 #include <gnuastro-internal/options.h>
@@ -179,35 +182,60 @@ void *
 gal_options_check_version(struct argp_option *option, char *arg,
                           char *filename, size_t lineno, void *junk)
 {
-  /* Check if the given value is different from this version. */
-  if( strcmp(arg, PACKAGE_VERSION) )
-    {
-      /* Print an error message and abort.  */
-      error_at_line(EXIT_FAILURE, 0, filename, lineno, "version mis-match: "
-                    "you are running GNU Astronomy Utilities (Gnuastro) "
-                    "version `%s'. However, the `onlyversion' option is set "
-                    "to version `%s'.\n\n"
-                    "This was probably done for reproducibility. Therefore, "
-                    "manually removing, or changing, the option value might "
-                    "produce errors or unexpected results. It is thus "
-                    "strongly advised to build Gnuastro %s and re-run this "
-                    "command/script.\n\n"
-                    "You can download previously released tar-balls from the "
-                    "following URLs respectively:\n\n"
-                    "    Stable (version format: X.Y):      "
-                    "http://ftpmirror.gnu.org/gnuastro\n"
-                    "    Alpha  (version format: X.Y.A-B):  "
-                    "http://alpha.gnu.org/gnu/gnuastro\n\n"
-                    "Alternatively, you can clone Gnuastro, checkout the "
-                    "respective commit (from the version number), then "
-                    "bootstrap and build it. Please run the following "
-                    "command for more information:\n\n"
-                    "    $ info gnuastro \"Version controlled source\"\n",
-                    PACKAGE_VERSION, arg, arg);
+  char *str;
 
-      /* Just to avoid compiler warnings for unused variables. The program
-         will never reach this point! */
-      arg=filename=NULL; lineno=0; option=NULL; junk=NULL;
+  /* First see if we are reading or writing. */
+  if(lineno==-1)
+    {
+      /* `PACKAGE_VERSION' is a static/literal string, but the pointer
+         returned by this function will be freed, so we must allocate space
+         for it.
+
+         We didn't allocate and give this option a value when we read it
+         because it is redundant and much more likely for the option to
+         just be present (for a check in a reproduction pipeline for
+         example) than for it to be printed. So we don't want to waste
+         resources in allocating a redundant value. */
+      gal_checkset_allocate_copy(PACKAGE_VERSION, &str);
+      return str;
+    }
+
+  /* Check if the given value is different from this version. */
+  else
+    {
+      if(arg==NULL)
+        error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to fix "
+              "the problem. The value to `arg' is NULL", __func__,
+              PACKAGE_BUGREPORT);
+      else if( strcmp(arg, PACKAGE_VERSION) )
+        {
+          /* Print an error message and abort.  */
+          error_at_line(EXIT_FAILURE, 0, filename, lineno, "version "
+                        "mis-match: you are running GNU Astronomy Utilities "
+                        "(Gnuastro) version `%s'. However, the `onlyversion' "
+                        "option is set to version `%s'.\n\n"
+                        "This was probably done for reproducibility. "
+                        "Therefore, manually removing, or changing, the "
+                        "option value might produce errors or unexpected "
+                        "results. It is thus strongly advised to build "
+                        "Gnuastro %s and re-run this command/script.\n\n"
+                        "You can download previously released tar-balls "
+                        "from the following URLs respectively:\n\n"
+                        "    Stable (version format: X.Y):      "
+                        "http://ftpmirror.gnu.org/gnuastro\n"
+                        "    Alpha  (version format: X.Y.A-B):  "
+                        "http://alpha.gnu.org/gnu/gnuastro\n\n"
+                        "Alternatively, you can clone Gnuastro, checkout the "
+                        "respective commit (from the version number), then "
+                        "bootstrap and build it. Please run the following "
+                        "command for more information:\n\n"
+                        "    $ info gnuastro \"Version controlled source\"\n",
+                        PACKAGE_VERSION, arg, arg);
+
+          /* Just to avoid compiler warnings for unused variables. The program
+             will never reach this point! */
+          arg=filename=NULL; lineno=0; option=NULL; junk=NULL;
+        }
     }
   return NULL;
 }
@@ -266,15 +294,21 @@ gal_options_print_citation(struct argp_option *option, char *arg,
 
 
   /* Notice for acknowledgements. */
-  asprintf(&gnuastro_acknowledgement,
-           "Acknowledgement\n"
-           "---------------\n"
-           "This work was partly done using GNU Astronomy Utilities "
-           "(Gnuastro) version %s. Gnuastro is a generic package for "
-           "astronomical data manipulation and analysis which was "
-           "initially created and developed for research funded by the "
-           "Monbukagakusho (Japanese government) scholarship and ERC "
-           "advanced grant 339659-MUSICOS.", PACKAGE_VERSION);
+  if( asprintf(&gnuastro_acknowledgement,
+               "Acknowledgement\n"
+               "---------------\n"
+               "This work was partly done using GNU Astronomy Utilities "
+               "(Gnuastro, ascl.net/1801.009) version %s. Work on Gnuastro "
+               "has been funded by the Japanese Ministry of Education, "
+               "Culture, Sports, Science, and Technology (MEXT) scholarship "
+               "and its Grant-in-Aid for Scientific Research (21244012, "
+               "24253003), the European Research Council (ERC) advanced "
+               "grant 339659-MUSICOS, European Union’s Horizon 2020 research "
+               "and innovation programme under Marie Sklodowska-Curie grant "
+               "agreement No 721463 to the SUNDIAL ITN, and from the Spanish "
+               "Ministry of Economy and Competitiveness (MINECO) under grant "
+               "number AYA2016-76219-P. ", PACKAGE_VERSION)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   printf("%s\n", gnuastro_acknowledgement);
   free(gnuastro_acknowledgement);
 
@@ -306,6 +340,62 @@ gal_options_print_citation(struct argp_option *option, char *arg,
 
 
 
+void *
+gal_options_check_config(struct argp_option *option, char *arg,
+                          char *filename, size_t lineno, void *junk)
+{
+  char *str;
+
+  /* First see if we are reading or writing. */
+  if(lineno==-1)
+    {
+      gal_checkset_allocate_copy("1", &str);
+      return str;
+    }
+
+  /* Check if the given value is different from this version. */
+  else
+    {
+      /* If its already set then ignore it. */
+      if(option->set) return NULL;
+
+      /* Activate the option and let the user know its activated. */
+      (*(uint8_t *)(option->value)) = 1;
+      printf("-----------------\n"
+             "Parsing of options AFTER `--checkconfig'.\n\n"
+             "IMPORTANT: Any option that was parsed before encountering "
+             "`--checkconfig', on the command-line or in a configuration "
+             "file, is not shown here. It is thus recommended to use this "
+             "option before calling any other option.\n"
+             "-----------------\n");
+
+      /* Print where this option was first seen: if `checkconfig' is called
+         within a configuration file, `filename!=NULL' and has an argument
+         (=="1"). But on the command-line, it has no argument or
+         filename. */
+      if(filename)
+        printf("%s:\n", filename);
+      else
+        {
+          if(arg)
+            error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+                  "fix the problem. `filename==NULL', but `arg!=NULL'",
+                  __func__, PACKAGE_BUGREPORT);
+          else
+            printf("Command-line:\n");
+        }
+
+      /* Just to avoid compiler warnings for unused variables. The program
+         will never reach this point! */
+      arg=filename=NULL; lineno=0; option=NULL; junk=NULL;
+    }
+  return NULL;
+}
+
+
+
+
+
 void *
 gal_options_read_type(struct argp_option *option, char *arg,
                       char *filename, size_t lineno, void *junk)
@@ -422,6 +512,55 @@ gal_options_read_tableformat(struct argp_option *option, char *arg,
 
 
 
+void *
+gal_options_read_interpmetric(struct argp_option *option, char *arg,
+                              char *filename, size_t lineno, void *junk)
+{
+  char *str;
+  if(lineno==-1)
+    {
+      switch(*(uint8_t *)(option->value))
+        {
+        case GAL_INTERPOLATE_CLOSE_METRIC_RADIAL:
+          gal_checkset_allocate_copy("radial", &str);
+          break;
+        case GAL_INTERPOLATE_CLOSE_METRIC_MANHATTAN:
+          gal_checkset_allocate_copy("manhattan", &str);
+          break;
+        default:
+          error(EXIT_FAILURE, 0, "%s: a bug! Please contact us at %s to "
+                "fix the problem. The code %u is not recognized as a "
+                "nearest-neighbor interpolation metric", __func__,
+                PACKAGE_BUGREPORT, *(uint8_t *)(option->value));
+        }
+      return str;
+    }
+  else
+    {
+      /* If the option is already set, just return. */
+      if(option->set) return NULL;
+
+      /* Set the value. */
+      if(       !strcmp(arg, "radial") )
+        *(uint8_t *)(option->value) = GAL_INTERPOLATE_CLOSE_METRIC_RADIAL;
+      else if ( !strcmp(arg, "manhattan") )
+        *(uint8_t *)(option->value) = GAL_INTERPOLATE_CLOSE_METRIC_MANHATTAN;
+      else
+        error_at_line(EXIT_FAILURE, 0, filename, lineno, "`%s' (value to "
+                      "`%s' option) isn't valid. Currently only `radial' "
+                      "and `manhattan' metrics are recognized for nearest "
+                      "neighbor interpolation", arg, option->name);
+
+      /* For no un-used variable warning. This function doesn't need the
+         pointer. */
+      return junk=NULL;
+    }
+}
+
+
+
+
+
 /* The input to this function is a string of any number of numbers
    separated by a comma (`,') and possibly containing fractions, for
    example: `1,2/3, 4.95'. The output `gal_data_t' contains the array of
@@ -456,6 +595,7 @@ gal_options_parse_list_of_numbers(char *string, char *filename, size_t lineno)
 
         /* Comma marks the transition to the next number. */
         case ',':
+        case ':':
           if(isnan(numerator))
             error_at_line(EXIT_FAILURE, 0, filename, lineno, "a number "
                           "must be given before `,'. You have given: `%s'",
@@ -788,7 +928,8 @@ gal_options_parse_sizes_reverse(struct argp_option *option, char *arg,
       /* Write the values into an allocated size_t array and finish it with
          a `-1' so the total number can be found later.*/
       num=values->size;
-      array=gal_data_malloc_array(GAL_TYPE_SIZE_T, num+1, __func__, "array");
+      array=gal_pointer_allocate(GAL_TYPE_SIZE_T, num+1, 0, __func__,
+                                 "array");
       for(i=0;i<num;++i) array[num-1-i]=v[i];
       array[num] = GAL_BLANK_SIZE_T;
 
@@ -877,7 +1018,8 @@ gal_options_read_sigma_clip(struct argp_option *option, char *arg,
   /* Caller wants to print the option values. */
   if(lineno==-1)
     {
-      asprintf(&str, "%g,%g", sigmaclip[0], sigmaclip[1]);
+      if( asprintf(&str, "%g,%g", sigmaclip[0], sigmaclip[1])<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
       return str;
     }
 
@@ -936,6 +1078,90 @@ gal_options_read_sigma_clip(struct argp_option *option, char *arg,
 
 
 
+/* Parse name and (float64) values:  name,value1,value2,value3,...
+
+   The output is a `gal_data_t', where the `name' is the given name and the
+   values are in its array (of `float64' type).
+ */
+void *
+gal_options_parse_name_and_values(struct argp_option *option, char *arg,
+                                  char *filename, size_t lineno, void *junk)
+{
+  size_t i, nc;
+  double *darray;
+  char *c, *name, *values;
+  gal_data_t *tmp, *existing, *dataset;
+  char *str, sstr[GAL_OPTIONS_STATIC_MEM_FOR_VALUES];
+
+  /* We want to print the stored values. */
+  if(lineno==-1)
+    {
+      /* Set the value pointer to `dataset'. */
+      existing=*(gal_data_t **)(option->value);
+      darray = existing->array;
+
+      /* First write the name. */
+      nc=0;
+      nc += sprintf(sstr+nc, "%s,", existing->name);
+
+      /* Write the values into a string. */
+      for(i=0;i<existing->size;++i)
+        {
+          if( nc > GAL_OPTIONS_STATIC_MEM_FOR_VALUES-100 )
+            error(EXIT_FAILURE, 0, "%s: a bug! please contact us at %s so we "
+                  "can address the problem. The number of necessary "
+                  "characters in the statically allocated string has become "
+                  "too close to %d", __func__, PACKAGE_BUGREPORT,
+                  GAL_OPTIONS_STATIC_MEM_FOR_VALUES);
+          nc += sprintf(sstr+nc, "%g,", darray[i]);
+        }
+      sstr[nc-1]='\0';
+
+      /* Copy the string into a dynamically allocated space, because it
+         will be freed later.*/
+      gal_checkset_allocate_copy(sstr, &str);
+      return str;
+    }
+  else
+    {
+      /* Parse until the comma or the end of the string.*/
+      c=arg; while(*c!='\0' && *c!=',') ++c;
+      values = (*c=='\0') ? NULL : c+1;
+
+      /* Name of the dataset (note that `c' is already pointing the end of
+         the `name' and `values' points to the next character). So we can
+         safely set `c' to `\0' to have the `name'. */
+      *c='\0';
+      gal_checkset_allocate_copy(arg, &name);
+
+      /* Read the values and write the name. */
+      dataset=gal_options_parse_list_of_numbers(values, filename, lineno);
+      dataset->name=name;
+
+      /* Add the given dataset to the end of an existing dataset. */
+      existing = *(gal_data_t **)(option->value);
+      if(existing)
+        {
+          for(tmp=existing;tmp!=NULL;tmp=tmp->next)
+            if(tmp->next==NULL) { tmp->next=dataset; break; }
+        }
+      else
+        *(gal_data_t **)(option->value) = dataset;
+
+      /* For a check.
+      printf("arg: %s\n", arg);
+      darray=dataset->array;
+      for(i=0;i<dataset->size;++i) printf("%f\n", darray[i]);
+      exit(0);
+      */
+
+      /* Our job is done, return NULL. */
+      return NULL;
+    }
+}
+
+
+
 
 
 
@@ -1111,11 +1337,11 @@ options_sanity_check(struct argp_option *option, char *arg,
      `GAL_ARITHMETIC_INPLACE' flags. But we will do this when there are
      multiple checks so from the two check data structures, we only have
      one remaining. */
-  check1=gal_arithmetic(operator1, GAL_ARITHMETIC_NUMOK, value, ref1);
+  check1=gal_arithmetic(operator1, 1, GAL_ARITHMETIC_NUMOK, value, ref1);
   if(ref2)
     {
-      check2=gal_arithmetic(operator2, GAL_ARITHMETIC_NUMOK, value, ref2);
-      check1=gal_arithmetic(multicheckop, mcflag, check1, check2);
+      check2=gal_arithmetic(operator2, 1, GAL_ARITHMETIC_NUMOK, value, ref2);
+      check1=gal_arithmetic(multicheckop, 1, mcflag, check1, check2);
     }
 
 
@@ -1169,6 +1395,12 @@ gal_options_read_check(struct argp_option *option, char *arg, char *filename,
          unset). */
       option->func(option, arg, filename, lineno, topass);
       if(option->key!=GAL_OPTIONS_KEY_CONFIG) option->set=GAL_OPTIONS_SET;
+
+      /* The `--config' option is printed for `--checkconfig' by its
+         function (`gal_options_call_parse_config_file'), so must be
+         ignored here. */
+      if(cp->checkconfig && option->key!=GAL_OPTIONS_KEY_CONFIG)
+        printf("  %-25s%s\n", option->name, arg?arg:"ACTIVATED");
       return;
     }
 
@@ -1182,7 +1414,12 @@ gal_options_read_check(struct argp_option *option, char *arg, char *filename,
       else
         {
           /* If the option is already set, ignore the given value. */
-          if(option->set==GAL_OPTIONS_SET) return;
+          if(option->set==GAL_OPTIONS_SET)
+            {
+              if(cp->checkconfig)
+                printf("  %-25s--ALREADY-SET--\n", option->name);
+              return;
+            }
 
           /* Read the string argument into the value. */
           if( gal_type_from_string(&option->value, arg, option->type) )
@@ -1210,7 +1447,12 @@ gal_options_read_check(struct argp_option *option, char *arg, char *filename,
   else
     {
       /* If the option is already set, ignore the given value. */
-      if(option->set==GAL_OPTIONS_SET) return;
+      if(option->set==GAL_OPTIONS_SET)
+        {
+          if(cp->checkconfig)
+            printf("  %-25s--ALREADY-SET--\n", option->name);
+          return;
+        }
 
       /* Make sure the option has the type set for options with no
          argument. So, give it a value of 1. */
@@ -1226,6 +1468,12 @@ gal_options_read_check(struct argp_option *option, char *arg, char *filename,
     }
 
 
+  /* If the user wanted to check the value. */
+  if(cp->checkconfig)
+    printf("  %-25s%s\n", option->name,
+           (arg && option->type!=GAL_OPTIONS_NO_ARG_TYPE)?arg:"ACTIVATED");
+
+
   /* Flip the `set' flag to `GAL_OPTIONS_SET'. */
   option->set=GAL_OPTIONS_SET;
 }
@@ -1328,6 +1576,50 @@ gal_options_common_argp_parse(int key, char *arg, struct argp_state *state)
 
 
 
+char *
+gal_options_stdin_error(long stdintimeout, int precedence, char *name)
+{
+  char *out;
+
+  if( asprintf(&out, "no %s!\n\n"
+               "The %s can be read from a file (specified as an argument), "
+               "or the standard input.%s Standard input can come from a "
+               "pipe (output of another program) or typed on the "
+               "command-line before %ld micro-seconds (configurable with "
+               "the `--stdintimeout' option).", name, name,
+               ( precedence
+                 ? " If both are provided, a file takes precedence."
+                 : "" ), stdintimeout )<0 )
+    error(EXIT_FAILURE, 0, "%s: `asprintf' allocation error", __func__);
+
+  return out;
+}
+
+
+
+
+
+/* Make the notice that is printed before program terminates, when no input
+   is given and Standard input is also available. */
+gal_list_str_t *
+gal_options_check_stdin(char *inputname, long stdintimeout, char *name)
+{
+  gal_list_str_t *lines=inputname ? NULL : gal_txt_stdin_read(stdintimeout);
+
+  /* See if atleast one of the two inputs is given. */
+  if(inputname==NULL && lines==NULL)
+    error( EXIT_FAILURE, 0, "%s", gal_options_stdin_error(stdintimeout,
+                                                          1, name));
+
+  /* Return the output. */
+  return lines;
+}
+
+
+
+
+
+
 
 
 
@@ -1458,7 +1750,13 @@ options_set_from_name(char *name, char *arg,  struct argp_option *options,
           if( options[i].flags==OPTION_HIDDEN
               || ( options[i].set
                    && !gal_type_is_list(options[i].type ) ) )
-            return 0;
+            {
+              if(cp->checkconfig)
+                printf("  %-25s%s\n", name, ( options[i].flags==OPTION_HIDDEN
+                                              ? "--IGNORED--"
+                                              : "--ALREADY-SET--" ) );
+              return 0;
+            }
 
           /* Read the value into the option and do a sanity check. */
           gal_options_read_check(&options[i], arg, filename, lineno, cp);
@@ -1517,18 +1815,16 @@ options_parse_file(char *filename,  struct gal_options_common_params *cp,
     return;
 
 
-  /* Open the file. If the file doesn't exist, then just ignore the
-     configuration file and return. */
+  /* Open the file. If the file doesn't exist or can't be opened, then just
+     ignore the configuration file and return. */
   errno=0;
   fp=fopen(filename, "r");
-  if(fp==NULL)
-    {
-      if(errno==ENOENT && enoent_abort==0)
-        return;
-      else
-        error(EXIT_FAILURE, errno, "%s: to be read as a configuration file",
-              filename);
-    }
+  if(fp==NULL) return;
+
+
+  /* If necessary, print the configuration file name. */
+  if(cp->checkconfig)
+    printf("%s:\n", filename);
 
 
   /* Allocate the space necessary to keep a copy of each line as we parse
@@ -1582,11 +1878,26 @@ options_parse_file(char *filename,  struct gal_options_common_params *cp,
 /* This function will be used when the `--config' option is called. */
 void *
 gal_options_call_parse_config_file(struct argp_option *option, char *arg,
-                                   char *filename, size_t lineno, void *cp)
+                                   char *filename, size_t lineno, void *c)
 {
+  struct gal_options_common_params *cp=(struct gal_options_common_params *)c;
+
+  /* The `--config' option is a special function when it comes to
+     `--checkconfig': we'll have to write its value before interpretting
+     it. */
+  if(cp->checkconfig)
+    {
+      printf("  %-25s%s\n", option->name, arg);
+      printf("............................\n");
+    }
+
   /* Call the confguration file parser. */
   options_parse_file(arg, cp, 1);
 
+  /* Ending boundary of this file's options. */
+  if(cp->checkconfig)
+      printf("............................\n");
+
   /* Just to avoid compiler warnings, then return, note that all pointers
      are just copies. */
   option=NULL; filename=NULL; lineno=0;
@@ -1623,12 +1934,14 @@ gal_options_parse_config_files(struct gal_options_common_params *cp)
           "`uint8' type", __func__, PACKAGE_BUGREPORT);
 
   /* The program's current directory configuration file. */
-  asprintf(&filename, ".%s/%s.conf", PACKAGE, cp->program_exec);
+  if( asprintf(&filename, ".%s/%s.conf", PACKAGE, cp->program_exec)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   options_parse_file(filename, cp, 0);
   free(filename);
 
   /* Common options configuration file. */
-  asprintf(&filename, ".%s/%s.conf", PACKAGE, PACKAGE);
+  if( asprintf(&filename, ".%s/%s.conf", PACKAGE, PACKAGE)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   options_parse_file(filename, cp, 0);
   free(filename);
 
@@ -1636,23 +1949,27 @@ gal_options_parse_config_files(struct gal_options_common_params *cp)
   home=options_get_home();
 
   /* The program's user-wide configuration file. */
-  asprintf(&filename, "%s/%s/%s.conf", home, USERCONFIG_DIR,
-           cp->program_exec);
+  if( asprintf(&filename, "%s/%s/%s.conf", home, USERCONFIG_DIR,
+               cp->program_exec)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   options_parse_file(filename, cp, 0);
   free(filename);
 
   /* Common options user-wide configuration file. */
-  asprintf(&filename, "%s/%s/%s.conf", home, USERCONFIG_DIR, PACKAGE);
+  if( asprintf(&filename, "%s/%s/%s.conf", home, USERCONFIG_DIR, PACKAGE)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   options_parse_file(filename, cp, 0);
   free(filename);
 
   /* The program's system-wide configuration file. */
-  asprintf(&filename, "%s/%s.conf", SYSCONFIG_DIR, cp->program_exec);
+  if( asprintf(&filename, "%s/%s.conf", SYSCONFIG_DIR, cp->program_exec)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   options_parse_file(filename, cp, 0);
   free(filename);
 
   /* Common options system-wide configuration file. */
-  asprintf(&filename, "%s/%s.conf", SYSCONFIG_DIR, PACKAGE);
+  if( asprintf(&filename, "%s/%s.conf", SYSCONFIG_DIR, PACKAGE)<0 )
+    error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
   options_parse_file(filename, cp, 0);
   free(filename);
 }
@@ -1685,6 +2002,51 @@ options_reverse_lists_check_mandatory(struct gal_options_common_params *cp,
 
 
 
+void
+gal_options_read_low_level_checks(struct gal_options_common_params *cp)
+{
+  size_t suggested_mmap=10000000;
+
+  /* If `numthreads' is 0, use the number of threads available to the
+     system. */
+  if(cp->numthreads==0)
+    cp->numthreads=gal_threads_number();
+
+  /* If `minmapsize==0' and quiet isn't given, print a warning. */
+  if(cp->minmapsize==0)
+    {
+      fprintf(stderr, "\n\n"
+              "========= WARNING =========\n"
+              "Minimum size to map an allocated space outside of RAM is "
+              "not set, or set to zero. This can greatly slow down the "
+              "processing of a program or cause strange crashes (recall "
+              "that the number of files that can be memory-mapped is "
+              "limited).\n\n"
+
+              "On modern systems (with RAM larger than a giga-byte), it "
+              "should be fine to set it to %zu (10 million bytes or 10Mb) "
+              "with the command below. In this manner, only arrays that "
+              "are larger than this will be memory-mapped and smaller "
+              "arrays (which are much more numerous) will be allocated and "
+              "freed in the RAM.\n\n"
+
+              "     --minmapsize=%zu\n\n"
+
+              "[This warning can be disabled with the `--quiet' (or `-q') "
+              "option.]\n"
+              "===========================\n\n", suggested_mmap,
+              suggested_mmap);
+    }
+
+  /* If the user wanted to check the parsing of configuration files, then
+     the program must stop here. */
+  if(cp->checkconfig) exit(0);
+}
+
+
+
+
+
 /* Read all configuration files and set common options */
 void
 gal_options_read_config_set(struct gal_options_common_params *cp)
@@ -1701,10 +2063,8 @@ gal_options_read_config_set(struct gal_options_common_params *cp)
   /* Abort if any of the mandatory options are not set. */
   gal_options_abort_if_mandatory_missing(cp);
 
-  /* If `numthreads' is 0, use the number of threads available to the
-     system. */
-  if(cp->numthreads==0)
-    cp->numthreads=gal_threads_number();
+  /* Low-level/basic checks before passing control back to program. */
+  gal_options_read_low_level_checks(cp);
 }
 
 
@@ -1976,6 +2336,7 @@ options_print_all(struct gal_options_common_params *cp, char *dirname,
 {
   size_t i;
   FILE *fp;
+  int errnum;
   time_t rawtime;
   char *topicstr, *filename;
   gal_list_i32_t *group=NULL;
@@ -1988,10 +2349,13 @@ options_print_all(struct gal_options_common_params *cp, char *dirname,
   if(dirname)
     {
       /* Make the host directory if it doesn't already exist. */
-      gal_checkset_mkdir(dirname);
+      if( (errnum=gal_checkset_mkdir(dirname)) )
+        error(EXIT_FAILURE, errnum, "making %s for configuration files",
+              dirname);
 
       /* Prepare the full filename: */
-      asprintf(&filename, "%s/%s.conf", dirname, cp->program_exec);
+      if( asprintf(&filename, "%s/%s.conf", dirname, cp->program_exec)<0 )
+        error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
 
       /* Remove the file if it already exists. */
       gal_checkset_writable_remove(filename, 0, 0);
@@ -2109,35 +2473,132 @@ gal_options_print_state(struct gal_options_common_params *cp)
         case GAL_OPTIONS_KEY_PRINTPARAMS:
         case GAL_OPTIONS_KEY_SETDIRCONF:
         case GAL_OPTIONS_KEY_SETUSRCONF:
+
+          /* Note that these options can have a value of 1 (enabled) or 0
+             (explicitly disabled). Therefore the printing should only be
+             done if they have a value of 1. This is why we have defined
+             the `OPTIONS_UINT8VAL' macro above. */
           sum += OPTIONS_UINT8VAL;
         }
-  if(sum>1)
-    error(EXIT_FAILURE, 0, "only one of the `printparams', `setdirconf' "
-          "and `setusrconf' options can be called in each run");
 
 
-  /* Print the required configuration files. Note that simply having a
-     non-NULL value is not enough. They can have a value of 1 or 0, and the
-     respective file should only be created if we have a value of 1. */
-  for(i=0; !gal_options_is_last(&cp->coptions[i]); ++i)
-    if(cp->coptions[i].set && OPTIONS_UINT8VAL)
-      switch(cp->coptions[i].key)
-        {
-        case GAL_OPTIONS_KEY_PRINTPARAMS:
-          options_print_all(cp, NULL, NULL);
-          break;
+  /* See if the values should be printed and if so, where. */
+  switch(sum)
+    {
+    /* No printing option has been called, so just return. */
+    case 0:  return;
 
-        case GAL_OPTIONS_KEY_SETDIRCONF:
-          asprintf(&dirname, ".%s", PACKAGE);
-          options_print_all(cp, dirname, cp->coptions[i].name);
-          free(dirname);
-          break;
+    /* (Only) one of the printing options has been called, so we'll need to
+       print the values in the proper place. */
+    case 1:
+      for(i=0; !gal_options_is_last(&cp->coptions[i]); ++i)
+        if(cp->coptions[i].set && OPTIONS_UINT8VAL)
+          switch(cp->coptions[i].key)
+            {
+            case GAL_OPTIONS_KEY_PRINTPARAMS:
+              options_print_all(cp, NULL, NULL);
+              break;
 
-        case GAL_OPTIONS_KEY_SETUSRCONF:
-          home=options_get_home();
-          asprintf(&dirname, "%s/%s", home, USERCONFIG_DIR);
-          options_print_all(cp, dirname, cp->coptions[i].name);
-          free(dirname);
-          break;
-        }
+            case GAL_OPTIONS_KEY_SETDIRCONF:
+              if( asprintf(&dirname, ".%s", PACKAGE)<0 )
+                error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+              options_print_all(cp, dirname, cp->coptions[i].name);
+              free(dirname);
+              break;
+
+            case GAL_OPTIONS_KEY_SETUSRCONF:
+              home=options_get_home();
+              if( asprintf(&dirname, "%s/%s", home, USERCONFIG_DIR)<0 )
+                error(EXIT_FAILURE, 0, "%s: asprintf allocation", __func__);
+              options_print_all(cp, dirname, cp->coptions[i].name);
+              free(dirname);
+              break;
+            }
+      break;
+
+    /* More than one of the printing options has been called. */
+    default:
+      error(EXIT_FAILURE, 0, "only one of the `printparams', `setdirconf' "
+            "and `setusrconf' options can be called in each run");
+    }
+}
+
+
+
+
+
+/* Main working function for common and program specific options. */
+static void
+options_as_fits_keywords_write(gal_fits_list_key_t **keys,
+                               struct argp_option *options,
+                               struct gal_options_common_params *cp)
+{
+  size_t i;
+  void *vptr;
+  uint8_t vtype;
+  char *name, *doc;
+  gal_list_str_t *tmp;
+
+  for(i=0; !gal_options_is_last(&options[i]); ++i)
+    if( options[i].set && option_is_printable(&options[i]) )
+      {
+        /* Linked lists (multiple calls to an option). */
+        if(gal_type_is_list(options[i].type))
+          for(tmp=*(gal_list_str_t **)(options[i].value);
+              tmp!=NULL; tmp=tmp->next)
+            {
+              /* `name' and `doc' have a `const' qualifier. */
+              gal_checkset_allocate_copy(options[i].name, &name);
+              gal_checkset_allocate_copy(options[i].doc,  &doc);
+              gal_fits_key_list_add(keys, GAL_TYPE_STRING, name, 1, tmp->v,
+                                    0, doc, 1, NULL);
+            }
+        /* Normal types. */
+        else
+          {
+            /* If the option is associated with a special function for
+               reading and writing, we'll need to write the value as a
+               string. */
+            if(options[i].func)
+              {
+                vtype=GAL_TYPE_STRING;
+                vptr=options[i].func(&options[i], NULL, NULL, (size_t)(-1),
+                                      cp->program_struct);
+              }
+            else
+              {
+                vtype=options[i].type;
+                vptr = ( vtype==GAL_TYPE_STRING
+                         ? *((char **)(options[i].value))
+                         : options[i].value );
+              }
+
+            /* Write the keyword. Note that `name' and `doc' have a `const'
+               qualifier. */
+            gal_checkset_allocate_copy(options[i].name, &name);
+            if(vtype==GAL_TYPE_STRING && strlen(vptr)>FLEN_KEYWORD)
+              gal_fits_key_write_filename(name, vptr, keys, 1);
+            else
+              {
+                gal_checkset_allocate_copy(options[i].doc,  &doc);
+                gal_fits_key_list_add(keys, vtype, name, 1, vptr, 0, doc, 1,
+                                      NULL);
+              }
+          }
+      }
+}
+
+
+
+
+/* Write all options as FITS keywords. */
+void
+gal_options_as_fits_keywords(struct gal_options_common_params *cp)
+{
+  /* Write all the command and program-specific options. */
+  options_as_fits_keywords_write(&cp->okeys, cp->coptions, cp);
+  options_as_fits_keywords_write(&cp->okeys, cp->poptions, cp);
+
+  /* Reverse the list (its a first-in-first-out list). */
+  gal_fits_key_list_reverse(&cp->okeys);
 }
