@@ -3,9 +3,9 @@ Functions to that only use WCSLIB functionality.
 This is part of GNU Astronomy Utilities (Gnuastro) package.
 
 Original author:
-     Mohammad Akhlaghi <akhlaghi@gnu.org>
+     Mohammad Akhlaghi <mohammad@akhlaghi.org>
 Contributing author(s):
-Copyright (C) 2015, Free Software Foundation, Inc.
+Copyright (C) 2015-2019, Free Software Foundation, Inc.
 
 Gnuastro is free software: you can redistribute it and/or modify it
 under the terms of the GNU General Public License as published by the
@@ -36,6 +36,7 @@ along with Gnuastro. If not, see <http://www.gnu.org/licenses/>.
 #include <gnuastro/wcs.h>
 #include <gnuastro/tile.h>
 #include <gnuastro/fits.h>
+#include <gnuastro/pointer.h>
 #include <gnuastro/dimension.h>
 #include <gnuastro/permutation.h>
 
@@ -183,6 +184,10 @@ gal_wcs_read(char *filename, char *hdu, size_t hstartwcs,
   fitsfile *fptr;
   struct wcsprm *wcs;
 
+  /* Make sure we are dealing with a FITS file. */
+  if( gal_fits_name_is_fits(filename) == 0 )
+    return NULL;
+
   /* Check HDU for realistic conditions: */
   fptr=gal_fits_hdu_open_format(filename, hdu, 0);
 
@@ -253,6 +258,177 @@ gal_wcs_copy(struct wcsprm *wcs)
 
 
 
+/* Remove the algorithm part of CTYPE (anything after, and including, a
+   `-') if necessary. */
+static void
+wcs_ctype_noalgorithm(char *str)
+{
+  size_t i, len=strlen(str);
+  for(i=0;i<len;++i) if(str[i]=='-') { str[i]='\0'; break; }
+}
+
+
+
+
+/* See if the CTYPE string ends with TAN. */
+static int
+wcs_ctype_has_tan(char *str)
+{
+  size_t len=strlen(str);
+
+  return !strcmp(&str[len-3], "TAN");
+}
+
+
+
+
+
+/* Remove dimension. */
+#define WCS_REMOVE_DIM_CHECK 0
+void
+gal_wcs_remove_dimension(struct wcsprm *wcs, size_t fitsdim)
+{
+  size_t c, i, j, naxis;
+
+  /* If the WCS structure is NULL, just return. */
+  if(wcs==NULL) return;
+
+  /* Sanity check. */
+  naxis=wcs->naxis;
+  if(fitsdim==0 || fitsdim>naxis)
+    error(EXIT_FAILURE, 0, "%s: requested dimension (fitsdim=%zu) must be "
+          "larger than zero and smaller than the number of dimensions in "
+          "the given WCS structure (%zu)", __func__, fitsdim, naxis);
+
+  /**************************************************/
+#if WCS_REMOVE_DIM_CHECK
+  printf("\n\nfitsdim: %zu\n", fitsdim);
+  printf("\n##################\n");
+  /*
+  wcs->pc[0]=0;   wcs->pc[1]=1;   wcs->pc[2]=2;
+  wcs->pc[3]=3;   wcs->pc[4]=4;   wcs->pc[5]=5;
+  wcs->pc[6]=6;   wcs->pc[7]=7;   wcs->pc[8]=8;
+  */
+  for(i=0;i<wcs->naxis;++i)
+    {
+      for(j=0;j<wcs->naxis;++j)
+        printf("%-5g", wcs->pc[i*wcs->naxis+j]);
+      printf("\n");
+    }
+#endif
+  /**************************************************/
+
+
+  /* First loop over the arrays. */
+  for(i=0;i<naxis;++i)
+    {
+      /* The dimensions are in FITS order, but counting starts from 0, so
+         we'll have to subtract 1 from `fitsdim'. */
+      if(i>fitsdim-1)
+        {
+          /* 1-D arrays. */
+          if(wcs->crpix) wcs->crpix[i-1] = wcs->crpix[i];
+          if(wcs->cdelt) wcs->cdelt[i-1] = wcs->cdelt[i];
+          if(wcs->crval) wcs->crval[i-1] = wcs->crval[i];
+          if(wcs->crota) wcs->crota[i-1] = wcs->crota[i];
+          if(wcs->crder) wcs->crder[i-1] = wcs->crder[i];
+          if(wcs->csyer) wcs->csyer[i-1] = wcs->csyer[i];
+
+          /* The strings are all statically allocated, so we don't need to
+             check. */
+          memcpy(wcs->cunit[i-1], wcs->cunit[i], 72);
+          memcpy(wcs->ctype[i-1], wcs->ctype[i], 72);
+          memcpy(wcs->cname[i-1], wcs->cname[i], 72);
+
+          /* For 2-D arrays, just bring up all the rows. We'll fix the
+             columns in a second loop. */
+          for(j=0;j<naxis;++j)
+            {
+              if(wcs->pc) wcs->pc[ (i-1)*naxis+j ] = wcs->pc[ i*naxis+j ];
+              if(wcs->cd) wcs->cd[ (i-1)*naxis+j ] = wcs->cd[ i*naxis+j ];
+            }
+        }
+    }
+
+
+  /**************************************************/
+#if WCS_REMOVE_DIM_CHECK
+  printf("\n###### Respective row removed (replaced).\n");
+  for(i=0;i<wcs->naxis;++i)
+    {
+      for(j=0;j<wcs->naxis;++j)
+        printf("%-5g", wcs->pc[i*wcs->naxis+j]);
+      printf("\n");
+    }
+#endif
+  /**************************************************/
+
+
+  /* Second loop for 2D arrays. */
+  c=0;
+  for(i=0;i<naxis;++i)
+    for(j=0;j<naxis;++j)
+      if(j!=fitsdim-1)
+        {
+          if(wcs->pc) wcs->pc[ c ] = wcs->pc[ i*naxis+j ];
+          if(wcs->cd) wcs->cd[ c ] = wcs->cd[ i*naxis+j ];
+          ++c;
+        }
+
+
+  /* Correct the total number of dimensions in the WCS structure. */
+  naxis = wcs->naxis -= 1;
+
+
+  /* The `TAN' algorithm needs two dimensions. So we need to remove it when
+     it can cause confusion. */
+  switch(naxis)
+    {
+    /* The `TAN' algorithm cannot be used for any single-dimensional
+       dataset. So we'll have to remove it if it exists. */
+    case 1:
+      wcs_ctype_noalgorithm(wcs->ctype[0]);
+      break;
+
+    /* For any other dimensionality, `TAN' should be kept only when exactly
+       two dimensions have it. */
+    default:
+
+      c=0;
+      for(i=0;i<naxis;++i)
+        if( wcs_ctype_has_tan(wcs->ctype[i]) )
+          ++c;
+
+      if(c!=2)
+        for(i=0;i<naxis;++i)
+          if( wcs_ctype_has_tan(wcs->ctype[i]) )
+            wcs_ctype_noalgorithm(wcs->ctype[i]);
+      break;
+    }
+
+
+
+  /**************************************************/
+#if WCS_REMOVE_DIM_CHECK
+  printf("\n###### Respective column removed.\n");
+  for(i=0;i<naxis;++i)
+    {
+      for(j=0;j<naxis;++j)
+        printf("%-5g", wcs->pc[i*naxis+j]);
+      printf("\n");
+    }
+  printf("\n###### One final string\n");
+  for(i=0;i<naxis;++i)
+    printf("%s\n", wcs->ctype[i]);
+  exit(0);
+#endif
+  /**************************************************/
+}
+
+
+
+
+
 /* Using the block data structure of the tile, add a WCS structure for
    it. In many cases, tiles are created for internal processing, so there
    is no need to keep their WCS. Hence for preformance reasons, when
@@ -265,8 +441,8 @@ gal_wcs_on_tile(gal_data_t *tile)
 {
   size_t i, start_ind, ndim=tile->ndim;
   gal_data_t *block=gal_tile_block(tile);
-  size_t *coord=gal_data_malloc_array(GAL_TYPE_SIZE_T, ndim, __func__,
-                                      "coord");
+  size_t *coord=gal_pointer_allocate(GAL_TYPE_SIZE_T, ndim, 0, __func__,
+                                     "coord");
 
   /* If the tile already has a WCS structure, don't do anything. */
   if(tile->wcs) return;
@@ -276,7 +452,8 @@ gal_wcs_on_tile(gal_data_t *tile)
       tile->wcs=gal_wcs_copy(block->wcs);
 
       /* Find the coordinates of the tile's starting index. */
-      start_ind=gal_data_ptr_dist(block->array, tile->array, block->type);
+      start_ind=gal_pointer_num_between(block->array, tile->array,
+                                        block->type);
       gal_dimension_index_to_coord(start_ind, ndim, block->dsize, coord);
 
       /* Correct the copied WCS structure. Note that crpix is indexed in
@@ -308,7 +485,7 @@ gal_wcs_on_tile(gal_data_t *tile)
 double *
 gal_wcs_warp_matrix(struct wcsprm *wcs)
 {
-  double *out;
+  double *out, crota2;
   size_t i, j, size=wcs->naxis*wcs->naxis;
 
   /* Allocate the necessary array. */
@@ -330,6 +507,33 @@ gal_wcs_warp_matrix(struct wcsprm *wcs)
       for(i=0;i<size;++i)
         out[i]=wcs->cd[i];
     }
+  else if(wcs->altlin & 0x4)     /* Has CROTAi array.   */
+    {
+      /* Basic sanity checks. */
+      if(wcs->naxis!=2)
+        error(EXIT_FAILURE, 0, "%s: CROTAi currently on works in 2 "
+              "dimensions.", __func__);
+      if(wcs->crota[0]!=0.0)
+        error(EXIT_FAILURE, 0, "%s: CROTA1 is not zero", __func__);
+
+      /* CROTAi keywords are depreciated in the FITS standard. However, old
+         files may still use them. For a full description of CROTAi
+         keywords and their history (along with the conversion equations
+         here), please see the link below:
+
+         https://fits.gsfc.nasa.gov/users_guide/users_guide/node57.html
+
+         Just note that the equations of the link above convert CROTAi to
+         PC. But here we want the "final" matrix (after multiplication by
+         the `CDELT' values). So to speed things up, we won't bother
+         dividing and then multiplying by the same CDELT values in the
+         off-diagonal elements. */
+      crota2=wcs->crota[1];
+      out[0] = wcs->cdelt[0] * cos(crota2);
+      out[1] = -1 * wcs->cdelt[1] *sin(crota2);
+      out[2] = wcs->cdelt[0] * sin(crota2);
+      out[3] = wcs->cdelt[1] * cos(crota2);
+    }
   else
     error(EXIT_FAILURE, 0, "%s: currently only PCi_ja and CDi_ja keywords "
           "are recognized", __func__);
@@ -360,11 +564,15 @@ gal_wcs_decompose_pc_cdelt(struct wcsprm *wcs)
   double *ps;
   size_t i, j;
 
+  /* If there is on WCS, then don't do anything. */
+  if(wcs==NULL) return;
+
   /* The correction is only needed when the PC matrix is filled. */
   if(wcs->pc)
     {
       /* Get the pixel scale. */
       ps=gal_wcs_pixel_scale(wcs);
+      if(ps==NULL) return;
 
       /* The PC matrix and the CDELT elements might both contain scale
          elements (during processing the scalings might be added only to PC
@@ -432,28 +640,94 @@ gal_wcs_angular_distance_deg(double r1, double d1, double r2, double d2)
 
 
 
-/* Return the pixel scale of the dataset in units of the WCS. */
+
 /* Return the pixel scale of the dataset in units of the WCS. */
 double *
 gal_wcs_pixel_scale(struct wcsprm *wcs)
 {
   gsl_vector S;
-  double *a, *out;
-  int permute_set;
   gsl_matrix A, V;
-  size_t i, j, n=wcs->naxis;
-  double *v=gal_data_malloc_array(GAL_TYPE_FLOAT64, n*n, __func__, "v");
-  size_t *permutation=gal_data_malloc_array(GAL_TYPE_SIZE_T, n, __func__,
-                                            "permutation");
-  gal_data_t *pixscale=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &n, NULL,
-                                      0, -1, NULL, NULL, NULL);
+  int warning_printed;
+  gal_data_t *pixscale;
+  size_t i, j, n, maxj, *permutation;
+  double jvmax, *a, *out, *v, maxrow, minrow;
 
+  /* Only continue if a WCS exists. */
+  if(wcs==NULL) return NULL;
 
   /* Write the full WCS rotation matrix into an array, irrespective of what
      style it was stored in the wcsprm structure (`PCi_j' style or `CDi_j'
      style). */
   a=gal_wcs_warp_matrix(wcs);
 
+  /* A small sanity check (this won't work on a singular matrix, can happen
+     in FITS WCSs!). In this case, we should return NULL.*/
+  n=wcs->naxis;
+  for(i=0;i<n;++i) {if(a[i*n+i]==0.0f) return NULL;}
+
+  /* Now that everything is good, we can allocate the necessary memory. */
+  v=gal_pointer_allocate(GAL_TYPE_FLOAT64, n*n, 0, __func__, "v");
+  permutation=gal_pointer_allocate(GAL_TYPE_SIZE_T, n, 0, __func__,
+                                   "permutation");
+  pixscale=gal_data_alloc(NULL, GAL_TYPE_FLOAT64, 1, &n, NULL,
+                          0, -1, NULL, NULL, NULL);
+
+
+  /* To avoid confusing issues with floating point errors being written in
+     the non-diagonal elements of the FITS header PC or CD matrices, we
+     need to check if the minimum and maximum values in each row are not
+     several orders of magnitude apart.
+
+     Note that in some cases (for example a spectrum), one axis might be in
+     degrees (value around 1e-5) and the other in angestroms (value around
+     1e-10). So we can't look at the minimum and maximum of the whole
+     matrix. However, in such cases, people will probably not warp/rotate
+     the image to mix the coordinates. So the important thing to check is
+     the minimum and maximum (non-zero) values in each row. */
+  warning_printed=0;
+  for(i=0;i<n;++i)
+    {
+      /* Find the minimum and maximum values in each row. */
+      minrow=FLT_MAX;
+      maxrow=-FLT_MAX;
+      for(j=0;j<n;++j)
+        if(a[i*n+j]!=0.0) /* We aren't concerned with 0 valued elements. */
+          {
+            /* We have to use absolutes because in cases like RA, the
+               diagonal values in different rows may have different signs*/
+            if(fabs(a[i*n+j])<minrow) minrow=fabs(a[i*n+j]);
+            if(fabs(a[i*n+j])>maxrow) maxrow=fabs(a[i*n+j]);
+          }
+
+      /* Do the check, print warning and make correction. */
+      if(maxrow!=minrow && maxrow/minrow>1e4 && warning_printed==0)
+        {
+          fprintf(stderr, "\nWARNING: The input WCS matrix (possibly taken "
+                  "from the FITS header keywords starting with `CD' or `PC') "
+                  "contains values with very different scales (more than "
+                  "10^4 different). This is probably due to floating point "
+                  "errors. These values might bias the pixel scale (and "
+                  "subsequent) calculations.\n\n"
+                  "You can see the respective matrix with one of the "
+                  "following two commands (depending on how the FITS file "
+                  "was written). Recall that if the desired extension/HDU "
+                  "isn't the default, you can choose it with the `--hdu' "
+                  "(or `-h') option before the `|' sign in these commands."
+                  "\n\n"
+                  "    $ astfits file.fits -p | grep 'PC._.'\n"
+                  "    $ astfits file.fits -p | grep 'CD._.'\n\n"
+                  "You can delete the ones with obvious floating point "
+                  "error values using the following command (assuming you "
+                  "want to delete `CD1_2' and `CD2_1'). Afterwards, you can "
+                  "rerun your original command to remove this warning "
+                  "message and possibly correct errors that it might have "
+                  "caused.\n\n"
+                  "    $ astfits file.fits --delete=CD1_2 --delete=CD2_1\n\n"
+                  );
+          warning_printed=1;
+        }
+    }
+
 
   /* Fill in the necessary GSL vector and Matrix structures. */
   S.size=n;     S.stride=1;                S.data=pixscale->array;
@@ -469,30 +743,28 @@ gal_wcs_pixel_scale(struct wcsprm *wcs)
 
   /* The raw pixel scale array produced from the singular value
      decomposition above is ordered based on values, not the input. So when
-     the pixel scales in all the dimensions aren't the same (for example in
-     IFU datacubes), the order of the values in `pixelscale' will not
-     necessarily correspond to the input's dimensions.
+     the pixel scales in all the dimensions aren't the same (the units of
+     the dimensions differ), the order of the values in `pixelscale' will
+     not necessarily correspond to the input's dimensions.
 
      To correct the order, we can use the `V' matrix to find the original
      position of the pixel scale values and then use permutation to
-     re-order it correspondingly. This works when there is only one
-     non-zero element in each row of `V'. */
+     re-order it correspondingly. The column with the largest (absolute)
+     value will be taken as the one to be used for each row. */
   for(i=0;i<n;++i)
     {
-      permute_set=0;
+      /* Find the column with the maximum value. */
+      maxj=-1;
+      jvmax=-FLT_MAX;
       for(j=0;j<n;++j)
-        if(v[i*n+j])
+        if(fabs(v[i*n+j])>jvmax)
           {
-            /* Only works when each row only has one non-zero value. */
-            if(permute_set)
-              error(EXIT_FAILURE, 0, "%s: not able to find the proper "
-                    "permutation for given rotation matrix", __func__);
-            else
-              {
-                permutation[i]=j;
-                permute_set=1;
-              }
+            maxj=j;
+            jvmax=fabs(v[i*n+j]);
           }
+
+      /* Use the column with the maximum value for this dimension. */
+      permutation[i]=maxj;
     }
 
 
@@ -525,12 +797,17 @@ gal_wcs_pixel_area_arcsec2(struct wcsprm *wcs)
   /* A small sanity check. Later, when higher dimensions are necessary, we
      can find which ones correlate to RA and Dec and use them to find the
      pixel area in arcsec^2. */
-  if(wcs->naxis!=2)
-    error(EXIT_FAILURE, 0, "%s: currently only 2D datasets supported. "
-          "The input WCS has %d dimensions", __func__, wcs->naxis);
+  if(wcs->naxis!=2) return NAN;
+
+  /* Check if the units of the axis are degrees or not. Currently all FITS
+     images I have worked with use `deg' for degrees. If other alternatives
+     exist, we can add corrections later. */
+  if( strcmp("deg", wcs->cunit[0]) || strcmp("deg", wcs->cunit[1]) )
+    return NAN;
 
   /* Get the pixel scales along each axis in degrees, then multiply. */
   pixscale=gal_wcs_pixel_scale(wcs);
+  if(pixscale==NULL) return NAN;
 
   /* Clean up and return the result. */
   out = pixscale[0] * pixscale[1] * 3600.0f * 3600.0f;
@@ -556,62 +833,172 @@ gal_wcs_pixel_area_arcsec2(struct wcsprm *wcs)
 
 
 
+
 /**************************************************************/
 /**********            Array conversion            ************/
 /**************************************************************/
-/* Convert an array of world coordinates to image coordinates. Note that in
-   Gnuastro, each column is treated independently, so the inputs are
-   separate. If `*x==NULL', or `*y==NULL', then space will be allocated for
-   them, otherwise, it is assumed that space has already been
-   allocated. Note that they must each be a 1 dimensional array.
-
-   You can do the conversion in place: just pass the same array as you give
-   to RA and Dec to X and Y. */
-void
-gal_wcs_world_to_img(struct wcsprm *wcs, double *ra, double *dec,
-                     double **x, double **y, size_t size)
+/* Some sanity checks for the WCS conversion functions. */
+static void
+wcs_convert_sanity_check_alloc(gal_data_t *coords, struct wcsprm *wcs,
+                               const char *func, int **stat, double **phi,
+                               double **theta, double **world,
+                               double **pixcrd, double **imgcrd)
 {
-  size_t i;
-  int status, *stat, ncoord=size, nelem=2;
-  double *phi, *theta, *world, *pixcrd, *imgcrd;
+  gal_data_t *tmp;
+  size_t ndim=0, firstsize=0, size=coords->size;
+
+  for(tmp=coords; tmp!=NULL; tmp=tmp->next)
+    {
+      /* Count how many coordinates are given. */
+      ++ndim;
+
+      /* Check the type of the input. */
+      if(tmp->type!=GAL_TYPE_FLOAT64)
+        error(EXIT_FAILURE, 0, "%s: input coordinates must have `float64' "
+              "type", func);
+
+      /* Make sure it has a single dimension. */
+      if(tmp->ndim!=1)
+        error(EXIT_FAILURE, 0, "%s: input coordinates for each dimension "
+              "must each be one dimensional. Coordinate dataset %zu of the "
+              "inputs has %zu dimensions", func, ndim, tmp->ndim);
+
+      /* See if all inputs have the same size. */
+      if(ndim==1) firstsize=tmp->size;
+      else
+        if(firstsize!=tmp->size)
+          error(EXIT_FAILURE, 0, "%s: all input coordinates must have the "
+                "same number of elements. Coordinate dataset %zu has %zu "
+                "elements while the first coordinate has %zu", func, ndim,
+                tmp->size, firstsize);
+    }
+
+  /* See if the number of coordinates given corresponds to the dimensions
+     of the WCS structure. */
+  if(ndim!=wcs->naxis)
+    error(EXIT_FAILURE, 0, "%s: the number of input coordinates (%zu) does "
+          "not match the dimensions of the input WCS structure (%d)", func,
+          ndim, wcs->naxis);
 
   /* Allocate all the necessary arrays. */
-  phi    = gal_data_malloc_array( GAL_TYPE_FLOAT64, size, __func__, "phi");
-  stat   = gal_data_calloc_array( GAL_TYPE_INT32,   size, __func__, "stat");
-  theta  = gal_data_malloc_array( GAL_TYPE_FLOAT64, size, __func__, "theta");
-  world  = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
+  *phi    = gal_pointer_allocate( GAL_TYPE_FLOAT64, size,      0, __func__,
+                                  "phi");
+  *stat   = gal_pointer_allocate( GAL_TYPE_INT32,   size,      1, __func__,
+                                  "stat");
+  *theta  = gal_pointer_allocate( GAL_TYPE_FLOAT64, size,      0, __func__,
+                                  "theta");
+  *world  = gal_pointer_allocate( GAL_TYPE_FLOAT64, ndim*size, 0, __func__,
                                   "world");
-  imgcrd = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
+  *imgcrd = gal_pointer_allocate( GAL_TYPE_FLOAT64, ndim*size, 0, __func__,
                                   "imgcrd");
-  pixcrd = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
+  *pixcrd = gal_pointer_allocate( GAL_TYPE_FLOAT64, ndim*size, 0, __func__,
                                   "pixcrd");
+}
 
-  /* Write the values into the allocated contiguous array. */
-  for(i=0;i<size;++i) { world[i*2]=ra[i]; world[i*2+1]=dec[i]; }
 
-  /* Use WCSLIB's `wcss2p'. */
+
+
+
+/* In Gnuastro, each column (coordinate for WCS conversion) is treated as a
+   separate array in a `gal_data_t' that are linked through a linked
+   list. But in WCSLIB, the input is a single array (with multiple
+   columns). This function will convert between the two. */
+static void
+wcs_convert_list_to_array(gal_data_t *list, double *array, int *stat,
+                          size_t ndim, int listtoarray)
+{
+  size_t i, d=0;
+  gal_data_t *tmp;
+
+  for(tmp=list; tmp!=NULL; tmp=tmp->next)
+    {
+      /* Put all this coordinate's values into the single array that is
+         input into or output from WCSLIB. */
+      for(i=0;i<list->size;++i)
+        {
+          if(listtoarray)
+            array[i*ndim+d] = ((double *)(tmp->array))[i];
+          else
+            ((double *)(tmp->array))[i] = stat[i] ? NAN : array[i*ndim+d];
+        }
+
+      /* Increment the dimension. */
+      ++d;
+    }
+}
+
+
+
+
+
+/* Prepare the output of the WCS conversion functions. */
+static gal_data_t *
+wcs_convert_prepare_out(gal_data_t *coords, struct wcsprm *wcs, int inplace)
+{
+  size_t i;
+  gal_data_t *out=NULL;
+  if(inplace)
+    out=coords;
+  else
+    for(i=0;i<wcs->naxis;++i)
+      gal_list_data_add_alloc(&out, NULL, GAL_TYPE_FLOAT64, 1,
+                              &coords->size, NULL, 0, coords->minmapsize,
+                              wcs->ctype[i], wcs->cunit[i], NULL);
+  return out;
+}
+
+
+
+
+
+/* Convert world coordinates to image coordinates given the input WCS
+   structure. The input must be a linked list of data structures of float64
+   (`double') type. The top element of the linked list must be the first
+   coordinate and etc. If `inplace' is non-zero, then the output will be
+   written into the input's allocated space. */
+gal_data_t *
+gal_wcs_world_to_img(gal_data_t *coords, struct wcsprm *wcs, int inplace)
+{
+  gal_data_t *out;
+  int status, *stat=NULL, ncoord=coords->size, nelem=wcs->naxis;
+  double *phi=NULL, *theta=NULL, *world=NULL, *pixcrd=NULL, *imgcrd=NULL;
+
+  /* Some sanity checks. */
+  wcs_convert_sanity_check_alloc(coords, wcs, __func__, &stat, &phi, &theta,
+                                 &world, &pixcrd, &imgcrd);
+
+
+  /* Write the values from the input list of separate columns into a single
+     array (WCSLIB input). */
+  wcs_convert_list_to_array(coords, world, stat, wcs->naxis, 1);
+
+
+  /* Use WCSLIB's wcsp2s for the conversion. */
   status=wcss2p(wcs, ncoord, nelem, world, phi, theta, imgcrd, pixcrd, stat);
   if(status)
     error(EXIT_FAILURE, 0, "%s: wcss2p ERROR %d: %s", __func__, status,
           wcs_errmsg[status]);
 
-  /* For a sanity check:
-  printf("\n\ngal_wcs_world_to_img sanity check:\n");
-  for(i=0;i<size;++i)
-    printf("world (%f, %f) --> pix (%f, %f), [stat: %d]\n",
-           world[i*2], world[i*2+1], pixcrd[i*2], pixcrd[i*2+1], stat[i]);
+
+  /* For a sanity check.
+  {
+    size_t i;
+    printf("\n\n%s sanity check:\n", __func__);
+    for(i=0;i<coords->size;++i)
+      printf("(%g, %g) --> (%g, %g), [stat: %d]\n", world[i*2], world[i*2+1],
+             pixcrd[i*2], pixcrd[i*2+1], stat[i]);
+  }
   */
 
-  /* Allocate the output arrays if they were not already allocated. */
-  if(*x==NULL) *x=gal_data_malloc_array(GAL_TYPE_FLOAT64, size, __func__,"x");
-  if(*y==NULL) *y=gal_data_malloc_array(GAL_TYPE_FLOAT64, size, __func__,"y");
 
-  /* Put the values into the output arrays. */
-  for(i=0;i<size;++i)
-    {
-      (*x)[i] = stat[i] ? NAN : pixcrd[i*2];
-      (*y)[i] = stat[i] ? NAN : pixcrd[i*2+1];
-    }
+  /* Allocate the output arrays if they were not already allocated. */
+  out=wcs_convert_prepare_out(coords, wcs, inplace);
+
+
+  /* Write the output from a single array (WCSLIB output) into the output
+     list of this function. */
+  wcs_convert_list_to_array(out, pixcrd, stat, wcs->naxis, 0);
+
 
   /* Clean up. */
   free(phi);
@@ -619,35 +1006,32 @@ gal_wcs_world_to_img(struct wcsprm *wcs, double *ra, double *dec,
   free(theta);
   free(world);
   free(pixcrd);
+
+  /* Return the output list of coordinates. */
+  return out;
 }
 
 
 
 
 
-/* Similar to `gal_wcs_world_to_img' but converts image coordinates into
-   world coordinates. */
-void
-gal_wcs_img_to_world(struct wcsprm *wcs, double *x, double *y,
-                     double **ra, double **dec, size_t size)
+/* Similar to `gal_wcs_world_to_img'. */
+gal_data_t *
+gal_wcs_img_to_world(gal_data_t *coords, struct wcsprm *wcs, int inplace)
 {
-  size_t i;
-  int status, *stat, ncoord=size, nelem=2;
-  double *phi, *theta, *world, *pixcrd, *imgcrd;
+  gal_data_t *out;
+  int status, *stat=NULL, ncoord=coords->size, nelem=wcs->naxis;
+  double *phi=NULL, *theta=NULL, *world=NULL, *pixcrd=NULL, *imgcrd=NULL;
 
-  /* Allocate all the necessary arrays. */
-  phi    = gal_data_malloc_array( GAL_TYPE_FLOAT64, size, __func__, "phi");
-  stat   = gal_data_calloc_array( GAL_TYPE_INT32,   size, __func__, "stat");
-  theta  = gal_data_malloc_array( GAL_TYPE_FLOAT64, size, __func__, "theta");
-  world  = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
-                                  "world");
-  imgcrd = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
-                                  "imgcrd");
-  pixcrd = gal_data_malloc_array( GAL_TYPE_FLOAT64, 2*size, __func__,
-                                  "pixcrd");
+  /* Some sanity checks. */
+  wcs_convert_sanity_check_alloc(coords, wcs, __func__, &stat, &phi, &theta,
+                                 &world, &pixcrd, &imgcrd);
+
+
+  /* Write the values from the input list of separate columns into a single
+     array (WCSLIB input). */
+  wcs_convert_list_to_array(coords, pixcrd, stat, wcs->naxis, 1);
 
-  /* Write the values into the allocated contiguous array. */
-  for(i=0;i<size;++i) { pixcrd[i*2]=x[i]; pixcrd[i*2+1]=y[i]; }
 
   /* Use WCSLIB's wcsp2s for the conversion. */
   status=wcsp2s(wcs, ncoord, nelem, pixcrd, imgcrd, phi, theta, world, stat);
@@ -655,25 +1039,26 @@ gal_wcs_img_to_world(struct wcsprm *wcs, double *x, double *y,
     error(EXIT_FAILURE, 0, "%s: wcsp2s ERROR %d: %s", __func__, status,
           wcs_errmsg[status]);
 
-  /* For a sanity check:
-  printf("\n\ngal_wcs_img_to_world sanity check:\n");
-  for(i=0;i<size;++i)
-    printf("img (%f, %f) --> world (%f, %f), [stat: %d]\n",
-           pixcrd[i*2], pixcrd[i*2+1], world[i*2], world[i*2+1], stat[i]);
+
+  /* For a sanity check.
+  {
+    size_t i;
+    printf("\n\n%s sanity check:\n", __func__);
+    for(i=0;i<coords->size;++i)
+      printf("(%g, %g) --> (%g, %g), [stat: %d]\n", pixcrd[i*2], pixcrd[i*2+1],
+             world[i*2],  world[i*2+1], stat[i]);
+  }
   */
 
-  /* Allocate the output arrays if they were not already allocated. */
-  if(*ra==NULL)
-    *ra  = gal_data_malloc_array(GAL_TYPE_FLOAT64, size, __func__, "ra");
-  if(*dec==NULL)
-    *dec = gal_data_malloc_array(GAL_TYPE_FLOAT64, size, __func__, "dec");
 
-  /* Put the values into the output arrays. */
-  for(i=0;i<size;++i)
-    {
-      (*ra)[i]  = stat[i] ? NAN : world[i*2];
-      (*dec)[i] = stat[i] ? NAN : world[i*2+1];
-    }
+  /* Allocate the output arrays if they were not already allocated. */
+  out=wcs_convert_prepare_out(coords, wcs, inplace);
+
+
+  /* Write the output from a single array (WCSLIB output) into the output
+     list of this function. */
+  wcs_convert_list_to_array(out, world, stat, wcs->naxis, 0);
+
 
   /* Clean up. */
   free(phi);
@@ -681,4 +1066,8 @@ gal_wcs_img_to_world(struct wcsprm *wcs, double *x, double *y,
   free(theta);
   free(world);
   free(pixcrd);
+
+
+  /* Return the output list of coordinates. */
+  return out;
 }
